package com.iver.andami.plugins;
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: Extension.java,v 1.6.4.2 2006/12/14 16:12:44 jmvivo Exp $
 * $Log: Extension.java,v $
 * Revision 1.6.4.2  2006/12/14 16:12:44  jmvivo
 * Modificaciones para dar soporte a que una Extension tenga el control exclusivo del UI (visibilidad/habilitado)
 *
 * Revision 1.6.4.1  2006/11/15 00:08:21  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.7  2006/09/15 10:39:18  caballero
 * multisplah y postInitialize
 *
 * Revision 1.6  2006/07/31 18:22:13  cesar
 * Rename finalize method to terminate method
 *
 * Revision 1.5  2006/05/02 15:53:06  jorpiell
 * Se ha cambiado la interfaz Extension por dos clases: una interfaz (IExtension) y una clase abstract(Extension). A partir de ahora todas las extensiones deben heredar de Extension
 *
 *
 */
/**
 * <p>
 * Una extensin es el mecanismo en la que los plugins pueden extender Andami.
 * En la aplicacin habr una instancia de cada extensin, las cuales tienen
 * unos mtodos que sern ejecutados en respuesta a acciones del usuario. Las
 * extensiones pueden aadir controles a la interfaz de usuario.  Pueden
 * aadir mens y herramientas de la barra de herramientas, de forma que al
 * ser seleccionadas se ejecute cdigo de la extensin. Adems de implementar esta
 * interfaz, una extensin ha de tener un constructor vaco, el cual ser invocado
 * para crear la instancia de la interfaz
 * </p>
 *
 * <p>
 * En un plugin correctamente implementado se crear una instancia de cada
 * extensin, que tiene que heredar de esta clase abstracta. Tras la creacin
 * se invocar al mtodo inicializar y luego con la aplicacin en marcha se
 * invocar a los mtodos isEnabled(), isVisible(), repetidas veces para
 * controlar la activacin y visualizacin de los controles relacionados
 * con la  extensin. Cuando se ejecuta una extensin se ejecuta el metodo
 * execute. Dicho mtodo se ejecuta en el thread de la interfaz de usuario,
 * por lo que dejar la interfaz bloqueada. Si se quiere dejar activa la
 * interfaz se deber lanzar un thread a parte. Al finalizar la extensin, se
 * ejecutar el mtodo finalize.
 * </p>
 *
 * <p>
 * Si se desea dotar de ms funcionalidad a las extensiones, se deben
 * incluir los nuevos mtodos en esta clase y sobreescribirlos en las
 * clases que implementen la extensin. De ese modo se evita que el impacto
 * sobre la aplicacin sea muy grande si se tiene que aadir un nuevo
 * mtodo en la extensin.
 * </p>
 *
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public abstract class Extension implements IExtension {

	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#terminate()
	 */
	public void terminate(){

	}
	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#postInitialize()
	 */
	public void postInitialize(){

	}

	/**
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#isEnabled(IExtension extension)
	 */
	public boolean isEnabled(IExtension extension) {		
		return extension.isEnabled();
	}

	/**
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#isVisible(IExtension extension)
	 */
	public boolean isVisible(IExtension extension) {
		return extension.isVisible();
	}

}

