package com.iver.andami.ui.splash;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;

import com.iver.andami.messages.Messages;
import com.iver.andami.ui.theme.Theme;

public class MultiSplashWindow extends JWindow implements TimerCallBack {

	private static final long serialVersionUID = 572592158521767258L;

	private Timer timer;
	private int numLogos = 0;
	private ImageIcon[] img = null;
	private static JProgressBar pb;
	private static JLabel lblStatus;
	private Dimension splashDimension;
	private int index = 0;
	private int current;
	private int lastIndex=-1;
	private Theme theme;
	private long[] timers;
	public MultiSplashWindow(Frame f, Theme theme) {
		super(f);
		this.theme=theme;
		this.timers=theme.getTimers();
		lblStatus = new JLabel(Messages.getString("SplashWindow.Iniciando"));
		lblStatus.setBorder(BorderFactory.createEtchedBorder());
		lblStatus.setBackground(Color.WHITE);

		pack();
		pb = new JProgressBar();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(lblStatus, BorderLayout.NORTH);
		getContentPane().add(pb, BorderLayout.SOUTH);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		init();
		setLocation((screenSize.width / 2) - (splashDimension.width / 2),
				(screenSize.height / 2) - (splashDimension.height / 2));
		index = 0;
		setVisible(true);

	}

	public void init(){
   	 img=theme.getSplashImages();
   	 numLogos=img.length;
   	 if (numLogos==0) {
   		 numLogos=1;
   		 img=new ImageIcon[1];
   		 img[0]=new ImageIcon(this.getClass().getClassLoader().getResource("images/logo_es.png"));
   		 timers=new long[1];
   		 timers[0]=1000l;
   	 }

   	 int splashWidth = img[0].getIconWidth();
   	 int splashHeight = img[0].getIconHeight();

   	 for (int i=1;i<numLogos;i++){
   		splashWidth = Math.max(splashWidth, img[i].getIconWidth());
		splashHeight = Math.max(splashHeight, img[i].getIconHeight());
   	 }
   	 splashDimension = new Dimension(splashWidth, splashHeight);
   	 setSize(splashDimension.width, splashDimension.height + 45);

   	 start();
    }

	public void paint(Graphics g) {
		if (lastIndex==current)
			return;
		super.paint(g);

		if (img == null || img[current] == null)
			return;
		ImageIcon image=img[current];
		g.drawImage(image.getImage(), (getWidth()/2)-(image.getIconWidth()/2),(getHeight()/2-image.getIconHeight()/2), this);
		lastIndex=current;
	}

	public void tick() {
		current = index;
		timer.setInterval(timers[current]);
		repaint();
		index++;
		if (index >= numLogos)
			index = 0;
	}

	public void start() {
		timer = new Timer(this, 1000);
		timer.start();
	}

	/**
	 * cierra la ventana
	 */
	public void close() {
		setVisible(false);
		stop();
		dispose();
	}

	public void stop() {
		timer.stop();
		//timer = null;
	}

	public static void main(String[] args) {
		Frame frame = new Frame();
		Theme theme=new Theme();
		MultiSplashWindow ba = new MultiSplashWindow(frame,theme);
		ba.setVisible(true);
		ba.setSize(500, 500);
		ba.init();
		ba.start();
	}

	/**
	 * @see com.iver.mdiApp.ui.AppLoadingListener#process(int)
	 */
	public static void process(int p, String str) {
		lblStatus.setText(str);
		// System.out.println("Porcentaje :" + p);
		if (pb.getValue() != p) {
			pb.setValue(p);
			pb.repaint();
			// pb.paint(pb.getGraphics());
		}
	}
}
