/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Component;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.DriverIOExceptionType;
import com.iver.cit.gvsig.fmap.DriverNotLoadedExceptionType;
import com.iver.cit.gvsig.fmap.GenericDriverExceptionType;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.project.documents.gui.FOpenDialog;
import com.iver.cit.gvsig.project.documents.gui.FileOpenDialog;
import com.iver.cit.gvsig.project.documents.view.gui.BaseView;
import com.iver.cit.gvsig.project.documents.view.gui.IView;


/**
 * Extensin que abre un dilogo para seleccionar la capa o capas que se quieren
 * aadir a la vista.
 *
 * @author Fernando Gonzlez Corts
 */
public class AddLayer extends Extension {
	public FOpenDialog fopen = null;

	private static ArrayList wizardStack = null;

	static {
		AddLayer.wizardStack = new ArrayList();
	}

	public static void addWizard(Class wpClass) {
		AddLayer.wizardStack.add(wpClass);
	}

	public static WizardPanel getInstance(int i)
			throws IllegalArgumentException, SecurityException,
			InstantiationException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {
		Class wpClass = (Class) AddLayer.wizardStack.get(i);
		Class[] args = {};
		Object[] params = {};
		WizardPanel wp = (WizardPanel) wpClass.getConstructor(args)
				.newInstance(params);

		wp.initWizard();

		return wp;
	}

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		com.iver.andami.ui.mdiManager.IWindow window = PluginServices.getMDIManager()
															 .getActiveWindow();

		if (window == null) {
			return false;
		}

		// Any view derived from BaseView should have AddLayer available

		IView view;
		try {
			view = (IView)window;
		}
		catch (ClassCastException e) {
		    return false;
		}

		if (view == null)
			return false;

		BaseView baseView = (BaseView)view;
		return (baseView != null);
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
	}

	private void checkProjection(FLayer lyr, ViewPort viewPort) {
		if (lyr instanceof FLayers){
			FLayers layers=(FLayers)lyr;
			for (int i=0;i<layers.getLayersCount();i++){
				checkProjection(layers.getLayer(i),viewPort);
			}
		}
		if (lyr instanceof FLyrVect) {
			FLyrVect lyrVect = (FLyrVect) lyr;
			IProjection proj = lyr.getProjection();
			// Comprobar que la projeccin es la misma que la vista
			if (proj == null) {
				// SUPONEMOS que la capa est en la proyeccin que
				// estamos pidiendo (que ya es mucho suponer, ya).
				lyrVect.setProjection(viewPort.getProjection());
				return;
			}
			if (proj != viewPort.getProjection()) {
				int option = JOptionPane.YES_OPTION;
				if (!CRSFactory.doesRigurousTransformations()) {
					option = JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(), PluginServices
						.getText(this, "reproyectar_aviso")+"\n"+ PluginServices.getText(this,"Capa")+": "+lyrVect.getName(), PluginServices
						.getText(this, "reproyectar_pregunta"),
						JOptionPane.YES_NO_OPTION);
				}

				if (option != JOptionPane.OK_OPTION) {
					return;
				}
				ICoordTrans ct = proj.getCT(viewPort.getProjection());
				lyrVect.setCoordTrans(ct);
				System.err.println("coordTrans = " + proj.getAbrev() + " "
						+ viewPort.getProjection().getAbrev());
			}
		}

	}

	/**
	 * @see com.iver.andami.plugins.IExtension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		// Project project = ((ProjectExtension)
		// PluginServices.getExtension(ProjectExtension.class)).getProject();
		BaseView theView = (BaseView) PluginServices.getMDIManager().getActiveWindow();
		MapControl mapControl=theView.getMapControl();
		this.addLayers(mapControl);
		mapControl.getMapContext().callLegendChanged();
		return;

	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
	}

	/**
	 * Creates FOpenDialog, and adds file tab, and additional registered tabs
	 *
	 * @return FOpenDialog
	 */
	private FOpenDialog createFOpenDialog() {
		fopen = new FOpenDialog();
		FileOpenDialog fileDlg = new FileOpenDialog(new Class[] {
				VectorialFileDriver.class, RasterDriver.class });
		// first, file wizard tab
		fopen.addTab(PluginServices.getText(this, "Fichero"), fileDlg);

		// after that, all registerez tabs (wizardpanels implementations)
		for (int i = 0; i < wizardStack.size(); i++) {
			WizardPanel wp;
			try {
				wp = AddLayer.getInstance(i);
				fopen.addWizardTab(wp.getTabName(), wp);
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}// for
		return fopen;
	}

	/**
	 * Adds to mapcontrol all the file based layers selected by user in
	 * fileOpenDialog  instance.
	 *
	 * @param mapControl
	 *            MapControl where we want to add the selected layers
	 *
	 * @param fileDlg
	 *            FileOpenDialog where user selected file based layers
	 *
	 * @return boolean flag to report sucess of the operation
	 */
	private boolean loadFileLayers(MapControl mapControl, FileOpenDialog fileDlg) {

		if (fileDlg.getFiles() == null) {
			return false;
		}

		FLayer lyr = null;
		IProjection proj = FOpenDialog.getLastProjection();
		File[] files = fileDlg.getFiles();
		String[] driverNames = fileDlg.getDriverNames();
		Driver[] drivers = new Driver[driverNames.length];

		//all catched errors will be saved here, to show user at the end
		//of the method
		ArrayList errors = new ArrayList();


		// try to load the drivers referenced by the file dialog
		for (int i = 0; i < drivers.length; i++) {
			try {
				drivers[i] = LayerFactory.getDM().getDriver(driverNames[i]);
			} catch (DriverLoadException e) {
				DriverNotLoadedExceptionType type = new DriverNotLoadedExceptionType();
				type.setDriverName(driverNames[i]);
				DriverException exception = new DriverException(e, type);
				errors.add(exception);
//				NotificationManager.addError("No se pudo cargar el driver", e);
			}
		}


		// Envelope de cada fichero seleccionado por el usuario
		Rectangle2D[] rects = new Rectangle2D[files.length];
		boolean first = false;

		// Aadir capas al mapControl se trata como una transaccion
		mapControl.getMapContext().beginAtomicEvent();

		for (int iFile = 0; iFile < files.length; iFile++) {
			File fich = files[iFile];
			String layerName = fich.getName();
			try {
// FJP: Comento esto (if (fileDlg.accept(fich))) para resolver
// el bug 75. Esto estaba antes
// o lo apuesto alguien por algo en concreto?.
// if (fileDlg.accept(fich))

				if (drivers[iFile] instanceof VectorialFileDriver) {
					lyr = LayerFactory.createLayer(layerName,
							(VectorialFileDriver) drivers[iFile], fich, proj);
				} else if (drivers[iFile] instanceof RasterDriver) {
					lyr = LayerFactory.createLayer(layerName,
							(RasterDriver) drivers[iFile], fich, proj);
				}

				if (lyr != null) {
					lyr.setVisible(true);
					if (mapControl.getMapContext().getViewPort().getExtent() == null) {
						first = true;
					}
					checkProjection(lyr, mapControl.getViewPort());
					mapControl.getMapContext().getLayers().addLayer(lyr);

	//esto ya se hace en layerfactory lo dejamos? (azabala)
	/*
	if (lyr instanceof FLyrVect) {
		FLyrVect lyrVect = (FLyrVect) lyr;
		if (drivers[iFile] instanceof WithDefaultLegend) {
			WithDefaultLegend aux = (WithDefaultLegend) drivers[iFile];
			lyrVect.setLegend((VectorialLegend) aux
					.getDefaultLegend());
		} else {
			lyrVect.setLegend(LegendFactory
					.createSingleSymbolLegend(lyrVect
							.getShapeType()));
		}
	}// if
	*/
					rects[iFile] = lyr.getFullExtent();

					// TODO: Poner una variable y dibujar solo cuando
					// todas las capas hayan sido cargadas.

					// TODO Se deber de redibujar mediante la captura de los
					// eventos, por
					// eso se comenta la parte anterior
					// theView.getMapControl().drawMap();
					// theView.getTOC().refresh();

				}// if

			} catch (DriverException e) {
				//This exception is produced in the layer.getFullExtent() call
				DriverIOExceptionType type =
					new DriverIOExceptionType();
				type.setFile(fich);
				DriverException newException = new DriverException(e, type);
				lyr.addError(newException);
				lyr.setAvailable(false);
//				NotificationManager.addError("Error al crear la capa", e);

				errors.add(newException);

			}


	/* si no seteamos la leyenda, pq ya se hace en el factory, esto quitarlo (azabala)
			catch (FieldNotFoundException e) {
				// Esta no puede saltar
			}
	*/


			catch (Exception e) {
//azabala: usamos el notificationManager?? a mi no me gusta demasiado
//NotificationManager.addError("Error al crear la capa", e);
				//we catch any possible error
				GenericDriverExceptionType type = new GenericDriverExceptionType();
				DriverException exception = new DriverException(e, type);
				lyr.addError(exception);
				lyr.setAvailable(false);

				errors.add(exception);
			}
		}// for



		//now, if it is the first time, we put the extent of the mapcontrol
		//to the global extent of all the loaded layers.
		if (rects.length > 1) {
			Rectangle2D rect = new Rectangle2D.Double();
			rect.setRect(rects[0]);

			//first is true only if the extent of the mapcontrol was null
			if (first) {
				for (int i = 0; i < rects.length; i++) {
					if (rects[i] != null) {
						rect.add(rects[i]);
					}
				}//for
				mapControl.getMapContext().getViewPort().setExtent(rect);
			}// if first
		}// if
		mapControl.getMapContext().endAtomicEvent();

		return true;
	}


	/**
	 * Adds to mapcontrol all layers selected by user in the specified WizardPanel.
	 *
	 * @param mapControl
	 * 	MapControl on which we want to load user selected layers.
	 * @param wizardPanel
	 * 	WizardPanel where user selected the layers to load
	 * @return
	 */
	private boolean loadGenericWizardPanelLayers(MapControl mapControl, WizardPanel wp) {
		FLayer lyr = null;
		wp.setMapCtrl(mapControl);
		wp.execute();
		lyr = wp.getLayer();
		
		if((lyr != null) && !(lyr.isOk())){
			//if the layer is not okay (it has errors) process them
			processErrorsOfLayer(lyr, mapControl);
		}

		if (lyr != null) {
			lyr.setVisible(true);
			mapControl.getMapContext().beginAtomicEvent();
			checkProjection(lyr, mapControl.getViewPort());
			mapControl.getMapContext().getLayers().addLayer(lyr);
			mapControl.getMapContext().endAtomicEvent();
			return true;
		}
		return false;
	}

	/**
	 * This method process the errors found in a layer
	 * @param lyr
	 * @param mapControl
	 */

	private void processErrorsOfLayer(FLayer lyr, MapControl mapControl){
		List errors = lyr.getErrors();
//		wp.callError(null);
		mapControl.getMapContext().callNewErrorEvent(null);
	}
	/**
	 * Abre dialogo para aadir capas y las aade en mapControl
	 *
	 * Devuelve true si se han aadido capas.
	 */
	public boolean addLayers(MapControl mapControl) {



		// create and show the modal fopen dialog
		fopen = createFOpenDialog();
		PluginServices.getMDIManager().addWindow(fopen);

		if (fopen.isAccepted()) {
			if (fopen.getSelectedTab() instanceof FileOpenDialog) {
				FileOpenDialog fileDlg = (FileOpenDialog) fopen
						.getSelectedTab();
				return loadFileLayers(mapControl, fileDlg);

			} else if (fopen.getSelectedTab() instanceof WizardPanel) {
				WizardPanel wp = (WizardPanel) fopen.getSelectedTab();
				return loadGenericWizardPanelLayers(mapControl, wp);
			} else {
				JOptionPane.showMessageDialog((Component) PluginServices
						.getMainFrame(), "Ninguna capa seleccionada");
			}
		}
		return false;
	}
}
