package com.iver.cit.gvsig.project.documents.layout;

import java.awt.Component;
import java.text.DateFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.documents.ProjectDocument;
import com.iver.cit.gvsig.project.documents.ProjectDocumentFactory;
import com.iver.cit.gvsig.project.documents.contextMenu.actions.CopyDocumentContextMenuAction;
import com.iver.cit.gvsig.project.documents.contextMenu.actions.CutDocumentContextMenuAction;
import com.iver.cit.gvsig.project.documents.contextMenu.actions.PasteDocumentContextMenuAction;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.cit.gvsig.project.documents.view.ProjectViewFactory;
import com.iver.utiles.XMLEntity;


/**
 * Factory of maps.
 *
 * @author Vicente Caballero Navarro
 */
public class ProjectMapFactory extends ProjectDocumentFactory {
    public static String registerName = "ProjectMap";

    /**
     * Returns image of button.
     *
     * @return Image button.
     */
    public ImageIcon getButtonIcon() {
        return new ImageIcon(ProjectMap.class.getClassLoader().getResource("images/mapas.png"));
    }

    /**
     * Returns image of selected button.
     *
     * @return Image button.
     */
    public ImageIcon getSelectedButtonIcon() {
        return new ImageIcon(ProjectMap.class.getClassLoader().getResource("images/mapas_sel.png"));
    }

    /**
     * Create a new ProjectDocument.
     *
     * @param project Opened project.
     *
     * @return ProjectDocument.
     */
    public ProjectDocument create(Project project) {
       String mapName = "";
    	String aux = PluginServices.getText(this, "untitled");
    	int numMaps=((Integer)ProjectDocument.NUMS.get(registerName)).intValue();
    	mapName = aux + " - " + numMaps++;
    	if (project != null) {
            while (existName(project, mapName)) {
                mapName = aux + " - " + numMaps++;
            }
        }
    	ProjectDocument.NUMS.put(registerName,new Integer(numMaps));
        ProjectMap map = createMap(mapName);
        map.setModel(new Layout());
        map.getModel().setProjectMap(map);
        map.setProject(project, 0);
        map.setProjectDocumentFactory(this);

        return map;
    }

    /**
     * Returns the name of registration in the point of extension.
     *
     * @return Name of registration
     */
    public String getRegisterName() {
        return registerName;
    }

    /**
     * Returns the name of ProjectDocument.
     *
     * @return Name of ProjectDocument.
     */
    public String getNameType() {
        return PluginServices.getText(this, "Mapa");
    }

    /**
     * Create a ne ProjectMap
     *
     * @param baseName name
     *
     * @return ProjectMap.
     */
    public static ProjectMap createMap(String baseName) {
       ProjectMap m = new ProjectMap();
        m.setName(baseName);
        m.setCreationDate(DateFormat.getInstance().format(new Date()));
        return m;
    }

    /**
     * Registers in the points of extension the Factory with alias.
     *
     */
    public static void register() {
        register(registerName, new ProjectMapFactory(),
            "com.iver.cit.gvsig.project.ProjectMap");

        registerAction(registerName,"copy",new CopyDocumentContextMenuAction());
        registerAction(registerName,"cut",new CutDocumentContextMenuAction());
        registerAction(registerName,"paste",new PasteDocumentContextMenuAction());
    }

    /**
     * Returns the priority of de ProjectDocument.
     *
     * @return Priority.
     */
    public int getPriority() {
        return 2;
    }

	public boolean resolveImportXMLConflicts(XMLEntity root, Project project, Hashtable conflicts) {
		XMLEntity xmlMaps = root.firstChild("type",this.getRegisterName());
		XMLEntity xmlViews = root.firstChild("type",ProjectViewFactory.registerName);
		XMLEntity xmlTables = root.firstChild("type",ProjectTableFactory.registerName);

		Hashtable mapsConflits = (Hashtable)conflicts.get(ProjectMapFactory.registerName);

		Hashtable viewsConflits = (Hashtable)conflicts.get(ProjectViewFactory.registerName);

		Hashtable tablesConflits = (Hashtable)conflicts.get(ProjectTableFactory.registerName);


		if (mapsConflits != null && mapsConflits.size() > 0) {
			int option = JOptionPane.showConfirmDialog(
					(Component)PluginServices.getMainFrame(),
					"<html>"+
						PluginServices.getText(this,"conflicto_de_nombres_de_mapas_al_pegar") + "<br>" +
						PluginServices.getText(this,"debera_introducir_nombres_para_los_mapas_a_pegar") + "<br>" +
					"</html>", //Mensaje
					PluginServices.getText(this,"pegar_mapas"),//titulo
					JOptionPane.YES_NO_OPTION
					);
			if (option != JOptionPane.YES_OPTION) {
				return false;
			}
			Enumeration en = mapsConflits.elements();
			while (en.hasMoreElements()) {
				XMLEntity map = (XMLEntity)en.nextElement();
				String newName = JOptionPane.showInputDialog(
						(Component)PluginServices.getMainFrame(),
						"<html>"+
							PluginServices.getText(this,"nuevo_nombre_para_el_mapa") +" "+  map.getStringProperty("name") + ":" +
					    "</html>", //Mensaje
						map.getStringProperty("name") //Valor por defecto
						);
				if (newName == null) {
					JOptionPane.showMessageDialog(
							(Component)PluginServices.getMainFrame(),
							"<html>"+PluginServices.getText(this,"operacion_cancelada")+"</html>",//Mensaje
							PluginServices.getText(this,"pegar_mapas"),//titulo
							JOptionPane.ERROR_MESSAGE
							);
				} else if (newName.equalsIgnoreCase(map.getStringProperty("name")) ) {
					JOptionPane.showMessageDialog(
							(Component)PluginServices.getMainFrame(),
							"<html>"+
								PluginServices.getText(this,"operacion_cancelada") +":<br>" +
								PluginServices.getText(this,"nombre_no_valido")+
							"</html>",//Mensaje
							PluginServices.getText(this,"pegar_mapas"),//titulo
							JOptionPane.ERROR_MESSAGE
							);
					return false;
				}
				map.setName(newName);
			}
		}

		if (viewsConflits != null && viewsConflits.size() > 0) {
			int option = JOptionPane.showConfirmDialog(
					(Component)PluginServices.getMainFrame(),
					"<html>"+
						PluginServices.getText(this,"conflicto_de_nombres_de_vistas_al_pegar") + "<br>" +
						PluginServices.getText(this,"no_se_pegaran_las_vistas_del_conflicto") + "<br>" +
						PluginServices.getText(this,"desea_continuar") +
					"</html>",
					PluginServices.getText(this,"pegar_mapas"),//titulo
					JOptionPane.YES_NO_OPTION
					);
			if (option != JOptionPane.YES_OPTION) {
				return false;
			}
			// Eliminamos las vistas del xml que no vamos a importar

			// Esto me devuelve los indices en orden inverso
			int[] indexes = this.getIndexOfConflict(viewsConflits);
			for (int i=0;i < indexes.length;i++) {
				xmlViews.removeChild(indexes[i]);
			}
			viewsConflits = null;

		}


		if (tablesConflits != null && tablesConflits.size() > 0) {
			int option = JOptionPane.showConfirmDialog(
					(Component)PluginServices.getMainFrame(),
					"<html>" +
						PluginServices.getText(this,"conflito_de_nombres_de_tablas_al_pegar") + "<br>" +
						PluginServices.getText(this,"no_se_pegaran_las_tablas") + "<br>" +
						PluginServices.getText(this,"desea_continuar") +
					"</html>", //Mensaje
					PluginServices.getText(this,"pegar_mapas"),
					JOptionPane.YES_NO_OPTION
					);
			if (option != JOptionPane.YES_OPTION) {
				return false;
			}
			xmlTables.removeAllChildren();
		}


		return true;
	}

	/**
	 * Devuelve las claves de conflits ordenados
	 * en orden inverso. Las claves se esperan que
	 * sean instancias de Integer
	 */
	private int[] getIndexOfConflict(Hashtable conflits) {
		Object[] tmpArray = conflits.keySet().toArray();
		Arrays.sort(tmpArray,new Comparator() {
			public int compare(Object o1, Object o2) {
				return ((Integer)o2).intValue() - ((Integer)o1).intValue();
			}
		}
		);
		int[] indexes = new int[] {tmpArray.length};
		for (int i = 0;i< tmpArray.length;i++) {
			indexes[i] = ((Integer)tmpArray[i]).intValue();
		}
		return indexes;


	}
}
