/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project.documents.view.toolListeners;


import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.text.NumberFormat;

import javax.swing.ImageIcon;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.CircleListener;


/**
 * Implementacin de la interfaz CircleListener como herramienta para
 * realizar una medicion circular
 *
 * @author Laura
 */
public class CircleMeasureListener implements CircleListener {
	private final Image img = new ImageIcon(MapControl.class.getResource(
				"images/RulerCursor.gif")).getImage();
	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(img,
			new Point(16, 16), "");
	private MapControl mapCtrl;

	/**
	 * Crea un nuevo AreaListenerImpl.
	 *
	 * @param mc MapControl.
	 */
	public CircleMeasureListener(MapControl mc) {
		this.mapCtrl = mc;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.CircleListener#circle(com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent)
	 */
	public void circle(MeasureEvent event) throws BehaviorException {
		double dist = 0;
		double distAll = 0;

		ViewPort vp = mapCtrl.getMapContext().getViewPort();

		for (int i = 0; i < (event.getXs().length - 1); i++) {
			dist = 0;

			Point2D p = new Point2D.Double(event.getXs()[i].doubleValue(),
					event.getYs()[i].doubleValue());
			Point2D p2 = new Point2D.Double(event.getXs()[i + 1].doubleValue(),
					event.getYs()[i + 1].doubleValue());

			///dist = vp.toMapDistance((int) p.distance(p2));
			dist = vp.distanceWorld(p, p2);
			distAll += dist;
		}
		//System.out.println("Distancia = " + dist + " Distancia Total = " +
		//	(distAll));
		NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        if (PluginServices.getMainFrame() != null)
        {
            PluginServices.getMainFrame().getStatusBar().setMessage("4",
    			"Dist:" + nf.format(dist/MapContext.CHANGEM[mapCtrl.getViewPort().getDistanceUnits()]) + "");
    		PluginServices.getMainFrame().getStatusBar().setMessage("5",
    			"Total:" + nf.format(distAll/MapContext.CHANGEM[mapCtrl.getViewPort().getDistanceUnits()]) + "");
        }
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}
}
