/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */
package es.prodevelop.cit.gvsig.arcims.gui.dialogs;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;

import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.StatusLayerRaster;

import es.prodevelop.cit.gvsig.arcims.fmap.drivers.FMapRasterArcImsDriver;
import es.prodevelop.cit.gvsig.arcims.fmap.layers.FRasterLyrArcIMS;

import org.cresques.cts.IProjection;

import org.cresques.filter.RasterFilterStackManager;
import org.cresques.filter.enhancement.BrightnessContrastStackManager;

import org.cresques.io.GeoRasterFile;

import org.cresques.px.Extent;
import org.cresques.px.PxRaster;

import org.cresques.ui.BrightnessContrast.EnhancedBrightnessContrastPanel;
import org.cresques.ui.filter.FilterRasterDialogPanel;
import org.cresques.ui.filter.RasterTransparencyPanel;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.InfoPanel;

import org.gvsig.remoteClient.arcims.ArcImsImageClient;
import org.gvsig.remoteClient.arcims.utils.ServiceInformation;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;

import java.io.File;

import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;


/**
 * <P>
 * Dialog for the raster properties of a ArcIMS layer.
 * It manages the avants and aplies
 * filters to the raster through the filter stack manager according to the user's
 * selection. This dialog contains some panels.
 * </P>
 * <UL>
 * <LI>Propierties</LI>
 * <LI>Band selection</LI>
 * <LI>Transparency</LI>
 * <LI>Enhancement</LI>
 * </UL>
 *
 */
public class ArcImsRasterPropsDialog extends FilterRasterDialogPanel
    implements IWindow, MouseListener, KeyListener, ActionListener {
    private static final int nprops = 13;
    private static final long serialVersionUID = 0;
    private IProjection currentProjection = null;
    private FRasterLyrArcIMS fLayer = null;
    private Object[][] props = null;
    private RasterFilterStackManager stackManager = null;
    private Status status = null;
    private StatusLayerRaster rasterStatus = null;
    private JFileChooser fileChooser = null;
    private String lastPath = new String("./");
    private PxRaster px = null;
    private GeoRasterFile grf = null;

    /**
     * Dialog window constructor.
     *
     * @param layer the layer
     * @param rangeR r band
     * @param rangeG g band
     * @param rangeB b band
     */
    public ArcImsRasterPropsDialog(FRasterLyrArcIMS layer, ArrayList ranges) {
        super();
        fLayer = layer;
        this.px = layer.getPxRaster();
        this.grf = layer.getGeoRasterFile();

        if (fLayer.getStatus() == null) {
            rasterStatus = new StatusLayerRaster();
            fLayer.setStatus(rasterStatus);
        }
        else
        {
            rasterStatus = (StatusLayerRaster) fLayer.getStatus();
        }

        init();
        this.setRanges(ranges);

        FilterRasterDialogPanel fr = ((FilterRasterDialogPanel) this.getContentPane());
        BandSetupPanel bandSetup = (BandSetupPanel) fr.getPanelByClassName(
                "BandSetupPanel");
        bandSetup.getFileList().getJButtonAdd().setEnabled(false);
        bandSetup.getFileList().getJButtonRemove().setEnabled(false);
        this.setTranslation();

        EnhancedBrightnessContrastPanel ep = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");
        ep.lstBrightness.getJSlider().addMouseListener(this);
        ep.lstContrast.getJSlider().addMouseListener(this);
        ep.getCheckSliderText().getJTextField().addKeyListener(this);
        ep.getLabelSliderText().getJTextField().addActionListener(this);
        ep.getLabelSliderText1().getJTextField().addActionListener(this);
    }

    /**
     * Dialog window constructor.
     */
    public ArcImsRasterPropsDialog() {
        init();
    }

    /**
     * Sets the panels' texts
     */
    private void setTranslation() {
        FilterRasterDialogPanel fr = ((FilterRasterDialogPanel) this.getContentPane());
        BandSetupPanel bandSetup = (BandSetupPanel) fr.getPanelByClassName(
                "BandSetupPanel");
        bandSetup.getFileList().getJButtonAdd()
                 .setText(PluginServices.getText(this, "add"));
        bandSetup.getFileList().getJButtonRemove()
                 .setText(PluginServices.getText(this, "remove"));
        bandSetup.getFileList().lbandasVisibles.setText(PluginServices.getText(
                this, "bands"));

        RasterTransparencyPanel tpan = (RasterTransparencyPanel) fr.getPanelByClassName(
                "RasterTransparencyPanel");

        tpan.getTransparencyCheck()
            .setText(PluginServices.getText(this, "transparencia"));
        tpan.getOpacityCheck().setText(PluginServices.getText(this, "opacidad"));

        EnhancedBrightnessContrastPanel ep = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        ep.lLineal.setText(PluginServices.getText(this, "lineal_directo"));
        ep.lRemove.setText(PluginServices.getText(this, "eliminar_extremos"));
        ep.cstEnhanced.setName(PluginServices.getText(this, "recorte_colas") +
            " ( % )");
        ep.lBrightC.setText(PluginServices.getText(this, "brillo_y_contraste"));
        ep.lstBrightness.setName(PluginServices.getText(this, "brillo"));
        ep.lstContrast.setName(PluginServices.getText(this, "contraste"));
        ep.lpreview.setText(PluginServices.getText(this, "previsualizacion"));
        ep.getPBrightCont()
          .setBorder(javax.swing.BorderFactory.createTitledBorder(
                javax.swing.BorderFactory.createLineBorder(
                    java.awt.Color.gray, 1),
                (PluginServices.getText(this, "brillo_y_contraste")),
                javax.swing.border.TitledBorder.LEFT,
                javax.swing.border.TitledBorder.DEFAULT_POSITION, null,
                java.awt.Color.black));
        ep.getPEnhanced()
          .setBorder(javax.swing.BorderFactory.createTitledBorder(
                javax.swing.BorderFactory.createLineBorder(
                    java.awt.Color.gray, 1),
                (PluginServices.getText(this, "realce")),
                javax.swing.border.TitledBorder.LEFT,
                javax.swing.border.TitledBorder.DEFAULT_POSITION, null,
                java.awt.Color.black));

        InfoPanel ip = (InfoPanel) super.getPanelByClassName("InfoPanel");

        ip.cabInfo = new String(PluginServices.getText(this, "Informacion"));
        ip.cabCoord = new String(PluginServices.getText(this, "coor_geograficas"));
        ip.cabProy = new String(PluginServices.getText(this, "Proyecciones"));
        ip.cabOrig = new String(PluginServices.getText(this, "origen_de_datos"));
        ip.cabMeta = new String(PluginServices.getText(this, "metadatos"));
        ip.refresh();

        //Recorremos los Tab y traducimos el nombre
        for (int i = 0; i < this.getTab().getTabCount(); i++) {
            if (this.getTab().getTitleAt(i).equals("Info")) {
                this.getTab().setTitleAt(i, PluginServices.getText(this, "info"));
            }

            if (this.getTab().getTitleAt(i).equals("Transparencia")) {
                this.getTab()
                    .setTitleAt(i, PluginServices.getText(this, "transparencia"));
            }

            if (this.getTab().getTitleAt(i).equals("Bands")) {
                this.getTab()
                    .setTitleAt(i, PluginServices.getText(this, "bands"));
            }

            if (this.getTab().getTitleAt(i).equals("Realce")) {
                this.getTab()
                    .setTitleAt(i, PluginServices.getText(this, "realce"));
            }
        }

        this.getAcceptButton().setText(PluginServices.getText(this, "ok"));
        this.getApplyButton().setText(PluginServices.getText(this, "apply"));
        this.getCancelButton().setText(PluginServices.getText(this, "cancel"));
    }

    /**
     * Assigns a FLayerRaster
     * @param layer        layer to be assigned
     */
    public void setFLyrWMS(FRasterLyrArcIMS layer) {
        fLayer = layer;
    }

    /**
     * Loads the info panel data.
     */
    private void loadInfoData() {
        if (fLayer != null) {
            props = new Object[nprops][2];
            props[0][0] = new String(PluginServices.getText(this, "Fichero"));
            props[0][1] = "http://" + fLayer.getHost().getHost() +
                fLayer.getHost().getFile();
            props[1][0] = new String(PluginServices.getText(this, "num_bandas"));
            props[1][1] = new String(String.valueOf(grf.getBandCount()));
            props[2][0] = new String(PluginServices.getText(this, "ancho_alto"));
            props[2][1] = (int) (fLayer.getWidth()) + " X " +
                (int) (fLayer.getHeight());
            props[3][0] = new String(PluginServices.getText(this, "format"));
            props[3][1] = fLayer.getFormat();
            props[4][0] = new String(PluginServices.getText(this, "tipo_dato"));

            String type = null;

            switch (grf.getDataType()) {
            case 0:
                type = new String("BYTE");

                break;

            case 1:
                type = new String("USHORT");

                break;

            case 2:
                type = new String("SHORT");

                break;

            case 3:
                type = new String("INT");

                break;

            case 4:
                type = new String("FLOAT");

                break;

            case 5:
                type = new String("DOUBLE");

                break;

            default:
                type = new String("UNDEFINED");

                break;
            }

            props[4][1] = type;
            props[5][0] = new String(PluginServices.getText(this,
                        "georreferenciado"));
            props[5][1] = new String(PluginServices.getText(this, "YES"));

            props[7][0] = new String(PluginServices.getText(this, "xmin"));
            props[7][1] = String.valueOf(px.getExtent().minX());
            props[8][0] = new String(PluginServices.getText(this, "ymin"));
            props[8][1] = String.valueOf(px.getExtent().minY());
            props[9][0] = new String(PluginServices.getText(this, "xmax"));
            props[9][1] = String.valueOf(px.getExtent().maxX());
            props[10][0] = new String(PluginServices.getText(this, "ymax"));
            props[10][1] = String.valueOf(px.getExtent().maxY());

            double tamRealX = fLayer.getMaxX() - fLayer.getMinX();
            double tamRealY = fLayer.getMaxY() - fLayer.getMinY();
            double tamX = tamRealX / fLayer.getWidth();
            double tamY = tamRealY / fLayer.getHeight();

            props[11][0] = new String(PluginServices.getText(this, "pixel_width"));
            props[11][1] = "" + tamX + " " +
                PluginServices.getText(this, "geounits"); // + units;
            props[12][0] = new String(PluginServices.getText(this,
                        "pixel_height"));
            props[12][1] = "" + tamY + " " +
                PluginServices.getText(this, "geounits"); // + units;
        }
        else {
            props = new Object[1][2];
            props[0][0] = new String("No props");
            props[0][1] = new String("-");
        }
    }

    /**
     * Adds bands to the FilterRasterDialogPanel's  band coounter
     * @param numBands Nmero de bandas a aadir
     */
    public void addNumBands(int numBands) {
        nbands += numBands;

        FilterRasterDialogPanel fr = ((FilterRasterDialogPanel) this.getContentPane());
        RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel) fr.getPanelByClassName(
                "RasterTransparencyPanel");

        if ((rasterTrans != null) &&
                rasterTrans.getPTranspByPixel().isControlEnabled()) {
            rasterTrans.setActiveTransparencyControl(true);
        }
    }

    /**
     * Inits the jDialog
     */
    public void init() {
        //this.setLayout(new FlowLayout());
        setName("filterRaster");

        this.loadInfoData();
        super.init(props);
        this.setTabVisible("Pansharpening", false);

        //this.add(getContentPane());
        this.setSize(486, 352);

        //contentPane.getAcceptButton().setEnabled(false);
        this.getAcceptButton().addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    acceptButtonActionPerformed(evt);
                    closeJDialog();
                }
            });
        this.getCancelButton().addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    cancelButtonActionPerformed(evt);
                    closeJDialog();
                }
            });
        this.getApplyButton().addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    acceptButtonActionPerformed(evt);
                }
            });

        BandSetupPanel bandSetup = (BandSetupPanel) ((FilterRasterDialogPanel) this.getContentPane()).getPanelByClassName(
                "BandSetupPanel");
        bandSetup.getFileList().getJButtonAdd().addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    addFileBand(evt);
                }
            });
        bandSetup.getFileList().getJButtonRemove().addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    delFileBand(evt);
                }
            });
    }

    /**
     * Saves the initial state to restore it if cancel.
     */
    public void readStat() {
        FilterRasterDialogPanel fr = ((FilterRasterDialogPanel) this.getContentPane());
        RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel) fr.getPanelByClassName(
                "RasterTransparencyPanel");
        status = new Status(rasterTrans.getOpacityText().getText(),
                getAssignedBand(GeoRasterFile.RED_BAND),
                getAssignedBand(GeoRasterFile.GREEN_BAND),
                getAssignedBand(GeoRasterFile.BLUE_BAND));
    }

    /**
     * This method initializes jContentPane
     */
    public Container getContentPane() {
        return this;
    }

    /**
     * Sets a projection.
     *
     * @param prj
     */
    public void setProjection(IProjection prj) {
        this.currentProjection = prj;
    }

    /**
     * Gets the projection.
     *
     * @return the projection
     */
    public IProjection getProjection() {
        return this.currentProjection;
    }

    public void closeJDialog() {
        PluginServices.getMDIManager().closeWindow(ArcImsRasterPropsDialog.this);
    }

    /**
     * Sets the RasterFilterStackManager
     * @param stackManager
     */
    public void setRasterFilterStackManager(
        RasterFilterStackManager stackManager) {
        this.stackManager = stackManager;
        stackManager.resetTempFilters();
    }

    /**
     *
     * @param flag the flag to be assigned
     *
     * @return the assigned band
     */
    public int getAssignedBand(int flag) {
        BandSetupPanel bandSetup = (BandSetupPanel) ((FilterRasterDialogPanel) this.getContentPane()).getPanelByClassName(
                "BandSetupPanel");

        return bandSetup.getAssignedBand(flag);
    }

    /**
     * Pressing OK with the bands panel selected makes its values to be
     * processed.
     *
     * @return true if the bands panel was selected and the action is processed, false it
     * was not selected.
     */
    public boolean processBandPanel() {
        fLayer.setBandR(getAssignedBand(GeoRasterFile.RED_BAND));
        fLayer.setBandG(getAssignedBand(GeoRasterFile.GREEN_BAND));
        fLayer.setBandB(getAssignedBand(GeoRasterFile.BLUE_BAND));
        rasterStatus.bandR = getAssignedBand(GeoRasterFile.RED_BAND);
        rasterStatus.bandG = getAssignedBand(GeoRasterFile.GREEN_BAND);
        rasterStatus.bandB = getAssignedBand(GeoRasterFile.BLUE_BAND);

        // Check whether there is any non-assigned band
        // and apply the filter
        StringBuffer sb = new StringBuffer();

        if (getAssignedBand(GeoRasterFile.RED_BAND) == -1) {
            sb.append("R");
        }

        if (getAssignedBand(GeoRasterFile.GREEN_BAND) == -1) {
            sb.append("G");
        }

        if (getAssignedBand(GeoRasterFile.BLUE_BAND) == -1) {
            sb.append("B");
        }

        if (!sb.toString().equals("")) {
            stackManager.addRemoveBands(sb.toString());
        }
        else
        {
            stackManager.removeFilter(stackManager.getTypeFilter("removebands"));
        }

        fLayer.getMapContext().invalidate();

        InfoPanel pInfo = (InfoPanel) super.getPanelByClassName("InfoPanel");
        pInfo.setBands(getAssignedBand(GeoRasterFile.RED_BAND),
            getAssignedBand(GeoRasterFile.GREEN_BAND),
            getAssignedBand(GeoRasterFile.BLUE_BAND));

        return true;
    }

    /**
     * Pressing OK with the transparency panel selected makes its values to be processed.
     * @return true if the transparency panel was selected and the action is processed, false it
     * was not selected.
     */
    public boolean processTransparencyPanel() {
        //OPACIDAD
        RasterTransparencyPanel tpan = (RasterTransparencyPanel) ((FilterRasterDialogPanel) this.getContentPane()).getPanelByClassName(
                "RasterTransparencyPanel");
        String sOpac = tpan.getOpacityText().getText();

        if (!sOpac.equals("") && tpan.getOpacityCheck().isSelected()) {
            int opac = Integer.parseInt(sOpac);
            opac = (int) ((opac * 255) / 100);
            px.setTransparency(true);

            //px.setTransparency(255-opac);
            fLayer.setTransparency(255 - opac);
            rasterStatus.transparency = 255 - opac;
        }
        else {
            px.setTransparency(false);
            fLayer.setTransparency(0);
            rasterStatus.transparency = 0;
        }

        //TRANSPARENCIA
        if (tpan.getTransparencyCheck().isSelected()) {
            stackManager.addTransparencyFilter(tpan.getPTranspByPixel()
                                                   .getEntries(), 0x10, //Transparencia
                0xff, //Color Transparencia R
                0xff, //Color Transparencia G
                0xff); //Color Transparencia B
        }
        else
        {
            px.filterStack.removeFilter(stackManager.getTypeFilter(
                    "transparency"));
        }

        fLayer.getMapContext().invalidate();

        return true;
    }

    private void processBrightnessContrastPanel() {
        EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        // Si est activo el panel de brillo y contraste tomamos los valores y cargamos un filtro de 
        // brillo y contraste
        BrightnessContrastStackManager bcStackManager = (BrightnessContrastStackManager) stackManager.getManagerByClass(BrightnessContrastStackManager.class);

        if (ebcPanel.getCBrightC().isSelected()) {
            int incrBrillo = (int) Math.round(Double.valueOf(
                        ebcPanel.lstBrightness.getTextValue()).doubleValue());
            int incrContraste = (int) Math.round(Double.valueOf(
                        ebcPanel.lstContrast.getTextValue()).doubleValue());
            bcStackManager.addBrightnessFilter(incrBrillo);
            bcStackManager.addContrastFilter(incrContraste);
        }
        else {
            stackManager.removeFilter(bcStackManager.brightness);
            stackManager.removeFilter(bcStackManager.contrast);
        }

        fLayer.getMapContext().invalidate();
    }

    /**
     * Pressing OK with the enhancement panel selected makes its values to be processed.
     * @return true if the enhancement panel was selected and the action is processed, false it
     * was not selected.
     */
    public void processEnhancedPanel() {
        EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        //Filtro de realce lineal seleccionado
        if (ebcPanel.getCEnhanced().isSelected()) {
            if ((ebcPanel.getJCheckBox().isSelected()) &&
                    (!ebcPanel.getCheckSliderText().getJCheckBox().isSelected())) {
                stackManager.addEnhancedFilter(true);
            }
            else
            {
                stackManager.addEnhancedFilter(false);
            }

            //Recorte de colas seleccionado
            if (ebcPanel.getCheckSliderText().getJCheckBox().isSelected()) {
                stackManager.removeFilter(stackManager.getTypeFilter(
                        "computeminmax"));

                double recorte = Double.parseDouble(ebcPanel.getCheckSliderText()
                                                            .getTextValue()) / 100;

                if (ebcPanel.getJCheckBox().isSelected()) {
                    stackManager.addTailFilter(recorte, 0D, true);
                }
                else
                {
                    stackManager.addTailFilter(recorte, 0D, false);
                }
            }
            else {
                stackManager.removeFilter(stackManager.getTypeFilter("tail"));
                stackManager.addComputeMinMaxFilter();
            }
        }

        // Sin filtro lineal seleccionado
        if (!ebcPanel.getCEnhanced().isSelected()) {
            stackManager.removeFilter(stackManager.getTypeFilter(
                    "computeminmax"));
            stackManager.removeFilter(stackManager.getTypeFilter("tail"));
            stackManager.removeFilter(stackManager.getTypeFilter("enhanced"));
        }

        fLayer.getMapContext().invalidate();
    }

    /**
     * Manages the action perfomed when Apply/OK is pressed or Apply at the raster
     * properties control.
     * @param e
     */
    private void acceptButtonActionPerformed(ActionEvent e) {
        processBandPanel();
        processTransparencyPanel();
        processEnhancedPanel();
        processBrightnessContrastPanel();
    }

    /**
     * Adds a band to the raster.
     * @param e
     */
    private void addFileBand(ActionEvent e) {
        String[] driverNames = null;
        String rasterDriver = null;

        //Creacin del dialogo para seleccin de ficheros
        fileChooser = new JFileChooser(lastPath);
        fileChooser.setMultiSelectionEnabled(true);
        fileChooser.setAcceptAllFileFilterUsed(false);

        try {
            driverNames = LayerFactory.getDM().getDriverNames();

            FileFilter defaultFileFilter = null;
            FileFilter auxF;

            for (int i = 0; i < driverNames.length; i++) {
                if (driverNames[i].endsWith("gvSIG Image Driver")) {
                    rasterDriver = driverNames[i];
                    auxF = new DriverFileFilter(driverNames[i]);
                    fileChooser.addChoosableFileFilter(auxF);
                    defaultFileFilter = auxF;
                }
            }
        }
        catch (DriverLoadException e1) {
            NotificationManager.addError("No se pudo acceder a los drivers", e1);
        }

        int result = fileChooser.showOpenDialog(ArcImsRasterPropsDialog.this);

        if (result == JFileChooser.APPROVE_OPTION) {
            File[] files = fileChooser.getSelectedFiles();
            FileFilter filter = fileChooser.getFileFilter();
            BandSetupPanel bandSetup = (BandSetupPanel) ((FilterRasterDialogPanel) this.getContentPane()).getPanelByClassName(
                    "BandSetupPanel");
            lastPath = files[0].getPath();

            //Lo aadimos a la capa si no esta
            Vector v = new Vector();

            for (int i = 0; i < files.length; i++) {
                boolean exist = false;

                for (int j = 0; j < px.getFiles().length; j++) {
                    if (px.getFiles()[j].getName().endsWith(files[i].getName())) {
                        exist = true;
                    }
                }

                if (!exist) {
                    try {
                        Rectangle2D extentOrigin = fLayer.getFullExtent();

                        Extent extentNewFile = GeoRasterFile.openFile(fLayer.getProjection(),
                                files[i].getAbsolutePath()).getExtent();

                        //Comprobamos que el extent y tamao del fichero aadido sea igual al 
                        //fichero original. Si no es as no abrimos la capa y mostramos un aviso
                        double widthNewFile = (extentNewFile.getMax().getX() -
                            extentNewFile.getMin().getX());
                        double heightNewFile = (extentNewFile.getMax().getY() -
                            extentNewFile.getMin().getY());

                        if (((widthNewFile - extentOrigin.getWidth()) > 1.0) ||
                                ((widthNewFile - extentOrigin.getWidth()) < -1.0) ||
                                ((heightNewFile - extentOrigin.getHeight()) > 1.0) ||
                                ((heightNewFile - extentOrigin.getHeight()) < -1.0)) {
                            JOptionPane.showMessageDialog(null,
                                PluginServices.getText(this,
                                    "extents_no_coincidentes"), "",
                                JOptionPane.ERROR_MESSAGE);

                            return;
                        }

                        if (((extentNewFile.getMax().getX() -
                                extentNewFile.getMin().getX()) != extentOrigin.getWidth()) ||
                                ((extentNewFile.getMax().getY() -
                                extentNewFile.getMin().getY()) != extentOrigin.getHeight())) {
                            JOptionPane.showMessageDialog(null,
                                PluginServices.getText(this,
                                    "extents_no_coincidentes"), "",
                                JOptionPane.ERROR_MESSAGE);

                            return;
                        }
                    }
                    catch (Exception exc) {
                        exc.printStackTrace();
                    }

                    //Lo aadimos a la capa
                    px.addFile(files[i].getAbsolutePath());
                }
                else {
                    JOptionPane.showMessageDialog(null,
                        PluginServices.getText(this, "fichero_existe") + " " +
                        files[i].getAbsolutePath(), "",
                        JOptionPane.ERROR_MESSAGE);
                }
            }

            //Aadimos los georasterfile a la tabla del Panel
            v = new Vector();

            Vector vInfo = new Vector();

            for (int i = 0; i < px.getFiles().length; i++) {
                boolean exist = false;

                for (int j = 0; j < bandSetup.getNBands(); j++) {
                    if (px.getFiles()[i].getName()
                                            .endsWith(bandSetup.getBandName(j))) {
                        exist = true;
                    }
                }

                if (!exist) {
                    v.add(px.getFiles()[i]);
                    vInfo.add(px.getFiles()[i]);
                }
            }

            GeoRasterFile[] grf = new GeoRasterFile[v.size()];

            for (int i = 0; i < grf.length; i++) {
                grf[i] = (GeoRasterFile) v.get(i);
            }

            GeoRasterFile[] infoGrf = new GeoRasterFile[vInfo.size()];

            for (int i = 0; i < infoGrf.length; i++) {
                infoGrf[i] = (GeoRasterFile) vInfo.get(i);
            }

            bandSetup.addFiles(grf);
            this.addInfoFiles(infoGrf);
        }
    }

    /**
     * Deletes a band from the raster. If there is only a file or no band is
     * selected then it does nothing.
     *
     * @param e
     */
    private void delFileBand(ActionEvent e) {
        BandSetupPanel bandSetup = (BandSetupPanel) ((FilterRasterDialogPanel) this.getContentPane()).getPanelByClassName(
                "BandSetupPanel");

        if ((bandSetup.getFileList().getJList().getSelectedValue() != null) &&
                (bandSetup.getFileList().getNFiles() > 1)) {
            String pathName = bandSetup.getFileList().getJList()
                                       .getSelectedValue().toString();
            px.delFile(pathName);

            String file = pathName.substring(pathName.lastIndexOf("/") + 1);
            file = file.substring(file.lastIndexOf("\\") + 1);
            bandSetup.removeFile(file);
        }

        Vector vInfo = new Vector();

        for (int i = 0; i < px.getFiles().length; i++) {
            vInfo.add(px.getFiles()[i]);
        }

        GeoRasterFile[] infoGrf = new GeoRasterFile[vInfo.size()];

        for (int i = 0; i < infoGrf.length; i++) {
            infoGrf[i] = (GeoRasterFile) vInfo.get(i);
        }

        this.addInfoFiles(infoGrf);
    }

    /**
     * The cancel button restores the previous state to the loading of this dialog
     * @param e        Evento
     */
    private void cancelButtonActionPerformed(ActionEvent e) {
        this.status.restoreStatus(this);
        fLayer.getMapContext().invalidate();
    }

    public WindowInfo getWindowInfo() {
        WindowInfo m_viewinfo = new WindowInfo(WindowInfo.MODALDIALOG |
                WindowInfo.RESIZABLE);
        m_viewinfo.setTitle(PluginServices.getText(this, "propiedades_raster"));

        return m_viewinfo;
    }

    public void actionPerformed(ActionEvent e) {
        EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        if ((e.getSource() == ebcPanel.getLabelSliderText().getJTextField()) ||
                (e.getSource() == ebcPanel.getLabelSliderText1().getJTextField())) {
            if (ebcPanel.getJCheckBox1().isSelected()) {
                processBrightnessContrastPanel();
            }
        }
    }

    public void mouseReleased(MouseEvent e) {
        EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        if (((e.getSource() == ebcPanel.lstBrightness.getJSlider()) ||
                (e.getSource() == ebcPanel.lstContrast.getJSlider())) &&
                (ebcPanel.getJCheckBox1().isSelected() == true)) {
            processBrightnessContrastPanel();
        }
    }

    public void keyPressed(KeyEvent e) {
        EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel) super.getPanelByClassName(
                "EnhancedBrightnessContrastPanel");

        if (e.getSource() == ebcPanel.getCheckSliderText().getJTextField()) {
            ebcPanel.getCheckSliderText().getJSlider()
                    .setValue((int) Math.round(Double.valueOf(
                        ebcPanel.getCheckSliderText().getTextValue())
                                                     .doubleValue()));
        }
    }

    public void mouseClicked(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    public void mouseEntered(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    public void mouseExited(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    public void mousePressed(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    public void keyReleased(KeyEvent arg0) {
        // TODO Auto-generated method stub
    }

    public void keyTyped(KeyEvent arg0) {
        // TODO Auto-generated method stub
    }

    /**
     * Class that holds the dialog state and restores the initial state if it
     * is cancelled.
     * @author Nacho Brodin <brodin_ign@gva.es>
     */
    class Status {
        public String inicAlpha;
        public int bandR;
        public int bandG;
        public int bandB;
        private ArrayList filters = new ArrayList();

        public Status(String alpha, int bandR, int bandG, int bandB) {
            this.inicAlpha = alpha;
            this.bandR = bandR;
            this.bandG = bandG;
            this.bandB = bandB;
            filters = stackManager.getStringsFromStack();
        }

        /**
         * Restaura el Estado salvado
         */
        public void restoreStatus(ArcImsRasterPropsDialog props) {
            // Reset the filter stack to the original state
            // if(stackManager != null) stackManager.deleteTempFilters();
            // Reset alpha
            int opac = Integer.parseInt(status.inicAlpha);
            opac = (int) ((opac * 255) / 100);
            fLayer.setTransparency(255 - opac);

            if (fLayer != null) {
                fLayer.setBandR(bandR);
                fLayer.setBandG(bandG);
                fLayer.setBandB(bandB);
            }

            // Restore filters
            if (filters != null) {
                stackManager.createStackFromStrings(filters);
            }

            fLayer.getMapContext().invalidate();
        }
    }

    public class DriverFileFilter extends FileFilter {
        private Driver driver;

        public DriverFileFilter(String driverName) throws DriverLoadException {
            driver = LayerFactory.getDM().getDriver(driverName);
        }

        /**
         * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
         */
        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }

            if (driver instanceof RasterDriver) {
                return ((RasterDriver) driver).fileAccepted(f);
            }
            else {
                throw new RuntimeException("Tipo no reconocido");
            }
        }

        /**
         * @see javax.swing.filechooser.FileFilter#getDescription()
         */
        public String getDescription() {
            return ((Driver) driver).getName();
        }
    }
}
