/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.listeners;

import java.awt.Component;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;

import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.GeorefPointerListenerImpl;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.project.documents.view.gui.View;


/**
 * <p>Extensin de la clase GeorefPointerListenerImpl de FMap para poder llamar a mtodos de
 * andami o de gvSIG.</p>
 * <p>Este listener salva puntos clickeados desde una vista sobre una capa de puntos.</p>
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeorefPointSelectorListener extends GeorefPointerListenerImpl {
	
	//**********************Vars**********************************
	private GeoreferencingDialog 	dialog = null;
	private View 					theView = null;
			
   /**
	* true si se va a seleccionar el primer punto y false si se selecciona el segundo.
	*/
	public static boolean 			firstPoint = true;
	
	/**
	 * Posicin del punto a ser seleccionado. 
	 */
	public static int 				posPoint = -1;
	//**********************End Vars******************************
	
	//**********************Methods*******************************
	/**
	 * Constructor
	 *
	 * @param mapCtrl
	 */
	public GeorefPointSelectorListener(GeoreferencingDialog dialog) {
		super();
		this.dialog = dialog;
	}

	/**
	 * Captura de puntos sobre una capa FlyrPoint
	 *
	 * @param event Evento de un punto sobre la vista
	 *
	 * @throws BehaviorException
	 */
	public void point(PointEvent event) throws BehaviorException {
		super.point(event);
			
		try{
			theView = (View)PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}
		//Obtenemos la capa de puntos y la capa de georaster
				
		if(dialog.getLyrPoints() == null || dialog.getLyrGeoRaster() == null){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
			return;
		}
		
		ViewPort viewPort = theView.getMapControl().getMapContext().getViewPort();
		Point2D wcPoint = viewPort.toMapPoint(event.getPoint());
					
		if(firstPoint){	
			if(dialog.getLyrPoints() != null){				
				//Obtenemos el punto seleccionado en coordenadas de la vista
				Point2D pixelImg = dialog.getLyrGeoRaster().world2Img(wcPoint);
				if(pixelImg == null){
					dialog.getPointManager().restoreControlsValue();
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
							PluginServices.getText(this, "fuera_de_extent"));
					return;
				}	
				
				dialog.getPointManager().selectFirstPoint(pixelImg);
			}
		}else
			dialog.getPointManager().selectSecondPoint(wcPoint);
			
	}
	
	//**********************End Methods*******************************
}
