package com.iver.cit.gvsig.fmap.layers;

import org.cresques.cts.IProjection;

import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DefaultDBDriver;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.drivers.WithDefaultLegend;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.utiles.XMLEntity;

public class FLayerJDBCVectorial  extends FLyrVect {
	private boolean loaded = false;

	private VectorialDatabaseDriver dbDriver = null;
	
	/* Esto deberia ir en el FLyrDefault */
	public void setProjectionByName(String projectionName) throws Exception{		
		IProjection proj = CRSFactory.getCRS(projectionName);
		if (proj == null) {
			throw new Exception("No se ha encontrado la proyeccion: "+ projectionName);
		}
		this.setProjection(proj);
		
	}

	
	public void setDriver(VectorialDatabaseDriver driver) {
		this.dbDriver = driver;
	}
	
	public void setDriverByName(String driverName) {
		this.setDriver(
		  (VectorialDatabaseDriver)LayerFactory.getDM().getDriver(driverName) 
		);
	}

	
	public VectorialDatabaseDriver getDriver() {
		return this.dbDriver;
	}
	/* FIXME: esto tendria que tener declarado un throws de algo*/
	public void wakeUp() {
		if (!loaded) {
			try {
				this.load();
			} catch (DriverIOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	
	public void load() throws DriverIOException {
		if (this.getName() == null || this.getName().length() == 0) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado nombre de capa");
		}
		if (this.dbDriver == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado driver");			
		}
		if (this.getProjection() == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado proyeccion");			
		}
				
		try {
			((DefaultDBDriver)this.dbDriver).load();
		} catch (Exception e) {
			this.setAvailable(false);
			throw new DriverIOException(e);	
		}
		
		try {
			VectorialDBAdapter dbAdapter = new VectorialDBAdapter();
			dbAdapter.setDriver(this.dbDriver);	
			this.setSource(dbAdapter);
		
		} catch (Exception e) {
			this.setAvailable(false);
			throw new DriverIOException(e);			
		}
		
		try {
			this.putLoadSelection();
			this.putLoadLegend();
			this.initializeLegendDefault();
			
		} catch (Exception e) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException(e);
		}
		this.cleanLoadOptions();
	}

	/* Esto deberia ir en FLyrVect */
	private void initializeLegendDefault() throws FieldNotFoundException, DriverException {
		if (this.getLegend() == null) {
            if (this.getRecordset().getDriver() instanceof WithDefaultLegend) {
                WithDefaultLegend aux = (WithDefaultLegend) this.getRecordset().getDriver();
                this.setLegend((VectorialLegend) aux.getDefaultLegend());                                        
            } else {
                this.setLegend(LegendFactory.createSingleSymbolLegend(
                        this.getShapeType()));
            }
		}		
	}
	
	public void setXMLEntity(XMLEntity xml) throws XMLException {
        IProjection proj = null;
        if (xml.contains("proj")) {
            proj = CRSFactory.getCRS(xml.getStringProperty("proj"));
        }
        else
        {
            proj = this.getMapContext().getViewPort().getProjection();
        }
		this.setName(xml.getName());
		this.setProjection(proj);
        String driverName = xml.getStringProperty("db");
        VectorialDatabaseDriver driver;
        try {     	
        
            driver = (VectorialDatabaseDriver) LayerFactory.getDM().getDriver(driverName);
            //Hay que separar la carga de los datos del XMLEntity del load.
            driver.setXMLEntity(xml.getChild(2));
            this.setDriver(driver);
            
        } catch (Exception e){
        	throw new XMLException(e);
        }
        super.setXMLEntityNew(xml);
        
	
	}

}
