/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.enhancement;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;

/**
 * Filtro de brillo que se aplica en la imagen. Toma como entrada la imagen
 * y el incremento de brillo y devuelve la imagen con lo nuevos valores de brillo.
 * @author Miguel ngel Querol Carratal  <querol_mig@gva.es>
 *
 */

public class BrightnessImageFilter extends BrightnessFilter {

	
	/**
	 * Constructor
	 *
	 */
	
	
	public BrightnessImageFilter(){
		super();
	}
	
	
	public void pre(){
		exec = true;
		this.image = (Image) params.get("raster");
		height = image.getHeight(null);
        width = image.getWidth(null);
		this.incrBrillo = ((Integer) params.get("incrBrillo")).intValue();
		
		super.pre();
	}
	
	public void process(int x, int y) {
		int px = ((BufferedImage) image).getRGB(x, y);
		int px3[] = {(px & 0xff000000) >> 24, 
					(px & 0x00ff0000) >> 16, 
					(px & 0x0000ff00) >> 8, 
					(px & 0x000000ff)};
		
		for (int i = 1 ; i < 4 ; i++){
			if((px3[i] + incrBrillo) > 255)
				px3[i] = 255;
			else if((px3[i] + incrBrillo) < 0)
				px3[i] = 0;
			else
				px3[i] += incrBrillo;
		}
		
		((BufferedImage) image).setRGB(x, y, 	((px3[0] << 24) & 0xff000000) | 
												((px3[1] << 16) & 0x00ff0000) |
												((px3[2] << 8) & 0x0000ff00) |
												(px3[3] & 0x000000ff));
	}

	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
	}

}
