/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.px;

import org.cresques.io.GeoFile;

import java.awt.Color;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;


/**
 * Paleta de color (en principio para MDT
 *
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class Paleta {
    public String fName = null;
    String name = null;
    public Color[] color = null;
    boolean enPaleta = false;

    /**
     * Lee una paleta desde un .lut de ArcView
     *
     * @param fName
     */
    public void loadLut(String fName) throws IOException {
        BufferedReader fi;
        String buf;
        int l = 0;

        if (fName == null) {
            fName = "C:/ESRI/AV_GIS30/ARCVIEW/averm/LUT/elevation.lut";
        }

        this.fName = fName;
        fi = new BufferedReader(new FileReader(fName));

        while ((buf = fi.readLine()) != null) {
            //System.out.println(buf);
            l++;
            parseLine(buf);
        }

        fi.close();

        //setColorFromLut("	  0     0 19200 65280");
    }

    private void parseLine(String buf) {
        //LookUpTable Begin
        //	Version         = "1.0"
        //	Name            = "elevation"
        //	Description     = "one of several std color schemes for color coding elevation data"
        //	NrEntries       = 256
        //	LUT             = {
        //	  0     0 19200 65280
        // ...
        //	}
        //LookUpTable End
        buf = GeoFile.filterWS(buf);

        String[] datos = buf.split(" ");
        System.out.println(buf);

        if (enPaleta) {
            if (buf.length() > 4) {
                int nr = Integer.parseInt(datos[0].trim());
                int r = Integer.parseInt(datos[1].trim());
                int g = Integer.parseInt(datos[2].trim());
                int b = Integer.parseInt(datos[3].trim());
                setColorFromLut(nr, r, g, b);
            } else {
                enPaleta = false;
            }
        } else if (buf.length() > 4) {
            if (buf.substring(0, 3).compareTo("LUT") == 0) {
                enPaleta = true;
            } else if (buf.substring(0, 3).compareTo("NrE") == 0) {
                int numC = Integer.parseInt(datos[2].trim());
                color = new Color[numC];
                System.out.println("hay " + numC + " colores.");
            }
        }
    }

    private void setColorFromLut(int nr, int r, int g, int b) {
        //"	  0     0 19200 65280"
        color[nr] = new Color(r / 256, g / 256, b / 256);

        //System.out.println(nr+":["+l.substring(5,10)+","+l.substring(11,16)+","+l.substring(17)+"]");	 
        System.out.println(nr + ":" + color[nr]);
    }
}
