/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cmd;

import org.cresques.px.Extent;

import org.cresques.ui.CQCursor;
import org.cresques.ui.CQMapCanvas;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;


/**
 * Comando zoom.
 * Aade al canvas la capacidad de aumentar y disminuir la vista.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class CmdZoom extends Cmd {
    public double aumenta = 2.0;
    public double reduce = .5;
    private Point2D ptIni = null;

    /**
     * Construye un nuevo CmdZoom para el Canvas
     * @param canvas
     */
    public CmdZoom(CQMapCanvas canvas) {
        super(canvas);
        eventsWanted = (LEFT | RIGHT | PRESS | RELEASE | DRAG);
        cursor = CQCursor.getCursor(CQCursor.ZOOMIN_CURSOR);
    }

    /**
     * Recibe los eventos del ratn.
     */
    public void cmd(Point2D pt, int bt, int mouseEvent) {
        //System.out.println("Zoom: Evento = "+ mouseEvent);
        if (mouseEvent == PRESS) {
            ptIni = pt;
        } else if (mouseEvent == RELEASE) {
            if (pt.getX() != ptIni.getX()) {
                zoomBox(new Extent(ptIni, pt));
            } else if (bt == MouseEvent.BUTTON1) {
                zoomMas(pt);
            } else if (bt == MouseEvent.BUTTON3) {
                zoomMenos(pt);
            }
        }
    }

    /**
     * Realiza un zoom de aumento, centrado en el Point2D;
     * @param pt
     */
    public void zoomMas(Point2D pt) {
        canvas.getVPData().zoom(aumenta, pt);
        canvas.viewPortChanged();
        canvas.repaint();
    }

    /**
     * Realiza un zoom de disminucin, centrado en el Point2D;
     * @param pt
     */
    public void zoomMenos(Point2D pt) {
        canvas.getVPData().zoom(reduce, pt);
        canvas.viewPortChanged();
        canvas.repaint();
    }

    /**
     * Realiza un zoom que abarque el Extent;
     * @param box
     */
    public void zoomBox(Extent box) {
        canvas.getVPData().zoom(box);
        canvas.viewPortChanged();
        canvas.repaint();
    }
}
