/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap;

import com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent;
import com.iver.cit.gvsig.fmap.layers.LayerEvent;
import com.iver.cit.gvsig.fmap.layers.LegendEvent;
import com.iver.cit.gvsig.fmap.layers.SelectionEvent;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Evento.
 */
public class AtomicEvent extends FMapEvent {
	private ArrayList events;

	/**
	 * Crea un nuevo AtomicEvent.
	 *
	 * @param fmapEvents ArrayList de FMapEvent.
	 */
	public AtomicEvent(ArrayList fmapEvents) {
		this.events = (ArrayList) fmapEvents.clone();
	}

	/**
	 * Devuelve el FMapEvent a partir de un ndice.
	 *
	 * @param index ndice.
	 *
	 * @return FMapEvent.
	 */
	public FMapEvent getEvent(int index) {
		return (FMapEvent) events.get(index);
	}

	/**
	 * Devuelve el nmero de eventos.
	 *
	 * @return Nmero de eventos.
	 */
	public int getEventCount() {
		return events.size();
	}

	/**
	 * Devuelve un vector con los eventos de leyenda.
	 *
	 * @return Vector de eventos de leyenda.
	 */
	public LegendEvent[] getLegendEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof LegendEvent) {
				ret.add(event);
			}
		}

		return (LegendEvent[]) ret.toArray(new LegendEvent[0]);
	}

	/**
	 * Devuelve un vector con los eventos de colecciones de capas.
	 *
	 * @return vector con los eventos de colecciones de capas.
	 */
	public LayerCollectionEvent[] getLayerCollectionEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof LayerCollectionEvent) {
				ret.add(event);
			}
		}

		return (LayerCollectionEvent[]) ret.toArray(new LayerCollectionEvent[0]);
	}

	/**
	 * Devuelve un vector con los eventos de seleccin.
	 *
	 * @return vector de eventos de seleccin.
	 */
	public SelectionEvent[] getSelectionEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof SelectionEvent) {
				ret.add(event);
			}
		}

		return (SelectionEvent[]) ret.toArray(new SelectionEvent[0]);
	}

	/**
	 * Devuelve un vector con los eventos de extent.
	 *
	 * @return vector con los eventos de extent.
	 */
	public ExtentEvent[] getExtentEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof ExtentEvent) {
				ret.add(event);
			}
		}

		return (ExtentEvent[]) ret.toArray(new ExtentEvent[0]);
	}

	/**
	 * Devuelve un vector con los eventos de capa.
	 *
	 * @return vector con los eventos de capa.
	 */
	public LayerEvent[] getLayerEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof LayerEvent) {
				ret.add(event);
			}
		}

		return (LayerEvent[]) ret.toArray(new LayerEvent[0]);
	}

	/**
	 * Devuelve un vector con los eventos de color.
	 *
	 * @return vector con los eventos de color.
	 */
	public ColorEvent[] getColorEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof ColorEvent) {
				ret.add(event);
			}
		}

		return (ColorEvent[]) ret.toArray(new ColorEvent[0]);
	}
	/**
	 * Devuelve un vector con los eventos de proyeccion.
	 *
	 * @return vector con los eventos de proyeccion.
	 */
	
	public ProjectionEvent[] getProjectionEvents() {
		ArrayList ret = new ArrayList();

		for (Iterator iter = events.iterator(); iter.hasNext();) {
			FMapEvent event = (FMapEvent) iter.next();

			if (event instanceof ProjectionEvent) {
				ret.add(event);
			}
		}

		return (ProjectionEvent[]) ret.toArray(new ProjectionEvent[0]);
	}

}
