/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.fmap.drivers;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.cresques.cts.ICoordTrans;
import org.cresques.filter.RasterFilterStack;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.data.Grid;
import org.cresques.io.data.RasterBuf;
import org.cresques.px.Extent;

import com.hardcode.driverManager.Driver;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.ISymbol;

public interface RasterDriver extends Driver {
	
	/**
	 * Set data buffer to create a memory raster driver
	 * @param buf data buffer
	 */
	public void setBuffer(RasterBuf buf);

	/**
	 * Set extent to create a memory raster driver
	 * @param ext Extent
	 */
	public void setExtent(Extent ext);
		
   /**
    * Define los tipos de fichero que puede leer el driver. Si devuelve true,
    * el fichero est aceptado (es de los que el driver puede leer), si
    * devuelve false es porque no lo puede leer.
    */
	public boolean fileAccepted(File fName);
	
	public void open(File f) throws IOException;

	public void initialize() throws IOException;

	public void close() throws IOException;

   /**
    * Obtiene la extensin en coordenadas del mundo real que ocupa este raster 
    */
	public Rectangle2D getFullExtent();
   /**
    * Obtiene el nmero de bandas del raster 
    */
	public int getNumBands();
	
	public int getTransparency();
	
	public void setTransparency(int trans);

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, ISymbol)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort)
			throws DriverIOException;

	/**
	 * Obtiene el valor que el raster tiene en la posicin x, y del bitmap
	 * @see com.iver.cit.gvsig.fmap.layers.RasterComplexOperations#getValueByCoords(int, int, int)
	 * /
	byte[] getPixel(int x, int y, byte[] dArray);

	/ **
	 * Obtiene el valor que el raster tiene en la posicin x, y del bitmap
	 * @see com.iver.cit.gvsig.fmap.layers.RasterComplexOperations#getValueByCoords(int, int, int)
	 * /
	int[] getPixel(int x, int y, int[] dArray);

	/**
	 * Obtiene el valor que el raster tiene en la posicin x, y del bitmap
	 * @see com.iver.cit.gvsig.fmap.layers.RasterComplexOperations#getValueByCoords(int, int, int)
	 * /
	float[] getPixel(int x, int y, float[] dArray);

	/**
	 * Obtiene el valor que el raster tiene en la posicin x, y del mundo real para la banda que se pasa como parmetro
	 * @see com.iver.cit.gvsig.fmap.layers.RasterComplexOperations#getValueByCoords(int, int, int)
	 * /
	double[] getPixel(int x, int y, double[] dArray);
	*/
	
	/**
	 * Devuelve el tipo de datos del raster
	 * @see java.awt.image.DataBuffer
	 */
	public int getRasterDataType();

   /**
    * Obtiene una muestra del pixel que se pasa como parmetro en la banda que se pasa como parmetro 
    */
	public Object getData(int x, int y, int band);

   /**
    * Obtiene una muestra del pixel que se pasa como parmetro en la banda que se pasa como parmetro 
    */
	public int getDataAsInt(int x, int y, int band);

   /**
    * Obtiene una muestra del pixel que se pasa como parmetro en la banda que se pasa como parmetro 
    */
	public byte getDataAsByte(int x, int y, int band);

   /**
    * Obtiene una muestra del pixel que se pasa como parmetro en la banda que se pasa como parmetro 
    */
	public float getDataAsFloat(int x, int y, int band);

   /**
    * Obtiene una muestra el pixel que se pasa como parmetro en la banda que se pasa como parmetro 
    */
	public double getDataAsDouble(int x, int y, int band);
	
	/**
	 * Obtiene una imagen de tamao especificado, de la zona (en
	 * coordenadas de usuario) que se le pide.
	 * @param size	Tamao (en pxeles de imgen).
	 * @param userSize Zona especificada (en coordenadas de usuario).
	 * @param rp Reproyeccin.
	 * @return La imagen.
	 */
	public Image getImage(Dimension size, Rectangle2D userSize, ICoordTrans rp);
	
	/**
	 * Obtiene un array con atributos de la imagen
	 * @return Lista de atributos
	 */
	public ArrayList getAttributes();
	
	/**
	 * Obtiene una banda
	 * @param flag	banda a recuperar
	 */
	public int getBand(int flag);
	
	/**
	 * Devuelve la posicin del fichero para la banda especificada.
	 * @param flag banda.
	 * @return posicin del fichero
	 */
	public int getPosFile(int flag);

	
	/**
	 * Activa o desactiva la transparencia
	 * @param t	true activa la transparencia y false la desactiva
	 */
	public void setTransparency(boolean t);
	
	/**
	 * Asocia un colorBand al rojo, verde o azul.
	 * @param flag cual (o cuales) de las bandas.
	 * @param nBand	que colorBand
	 */
	public void setBand(int flag, int nBand);
	
	/**
	 * Obtiene la pila de filtros
	 * @return	Pila de filtros
	 */
	public RasterFilterStack getFilterStack();
	
	/**
	 * Asigna la pila de filtros 
	 * @param pila de filtros
	 */
	public void setFilterStack(RasterFilterStack  stack);
	
	public GeoRasterFile [] getFiles();
	
	/**
	 * Asigna un fichero al PxRaster
	 * @param fileName Nombre del fichero
	 */
	public void addFile(String fileName);

	/**
	 * Elimina un fichero al PxRaster
	 * @param fileName Nombre del fichero
	 */
	public void delFile(String fileName);
	
	/**
	 * Obtiene el valor del pixel del Image en la posicin x,y
	 * @param x Posicin x
	 * @param y Posicin y
	 * @return valor de pixel
	 */
	public int[] getPixel(double wcx, double wcy);
		
	/**
	 * Obtiene el grid asociado
	 * @return grid
	 */
	public Grid getGrid();
			
}