package com.iver.cit.gvsig.fmap.drivers.gml;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JDialog;

import org.apache.log4j.Logger;
import org.gvsig.remoteClient.gml.GMLReader;
import org.gvsig.remoteClient.gml.IGMLFeaturesIterator;
import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.gml.exceptions.GMLExceptionList;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;

import com.hardcode.gdbms.engine.values.IntValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.ConcreteMemoryDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.drivers.gml.FMAPGeometryFactory.FeatureWithAttributes;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.VectorialUniqueValueLegend;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLDriver.java,v 1.2.4.6 2007/02/09 13:06:19 jorpiell Exp $
 * $Log: GMLDriver.java,v $
 * Revision 1.2.4.6  2007/02/09 13:06:19  jorpiell
 * Haba una exceptcin con algunas geometras que no se podan parsear
 *
 * Revision 1.2.4.5  2007/01/25 16:11:35  jorpiell
 * Se han cambiado los imports que hacan referencia a remoteServices. Esto es as, porque se han renombrado las clases del driver de GML
 *
 * Revision 1.9  2007/01/16 08:40:49  jorpiell
 * Eliminado un improt para hacer que compile
 *
 * Revision 1.8  2007/01/16 08:34:37  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.7  2006/12/29 17:12:10  jorpiell
 * Se tienen en cuenta los simpleTypes y los choices, adems de los atributos multiples
 *
 * Revision 1.6  2006/10/02 08:49:50  jorpiell
 * Cambios provocados el el cambio de FSymbol
 *
 * Revision 1.5  2006/10/02 08:41:23  jorpiell
 * Actualizados los drivers de GML
 *
 * Revision 1.2.4.2  2006/09/28 08:53:47  jorpiell
 * No se tiene en cuanta la reproyeccin
 *
 * Revision 1.2.4.1  2006/09/19 12:22:48  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.3  2006/09/18 12:09:43  jorpiell
 * El driver de GML ya no depende de geotools
 *
 * Revision 1.2  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 * Revision 1.1  2006/07/19 12:29:39  jorpiell
 * Aadido el driver de GML
 *
 *
 */
/**
 * This class implements the gvSIG GML parser using the geotools
 * library
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLDriver extends ConcreteMemoryDriver implements VectorialFileDriver{
	private static Logger logger = Logger.getLogger(GMLDriver.class.getName());
	private File m_Fich;	
	private IGeometriesFactory factory = new FMAPGeometryFactory();
	private Hashtable hashRelate = new Hashtable();
	private ArrayList features = new ArrayList();
	private IGMLFeaturesIterator iterator = null;
	private GMLReader reader;
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		m_Fich = f;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
		// TODO Auto-generated method stub
		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException{
		float heightText = 10;
		
		try {
			reader = new GMLReader(m_Fich,factory);
			iterator = reader.getFeaturesIterator();
		} catch (GMLException e1) {
			e1.printStackTrace();
			throw new IOException(e1.toString());
		}		
		
	
		FSymbol symbol = new FSymbol(getShapeType());
		symbol.setShapeVisible(false);
		symbol.setFontSizeInPixels(false);
		symbol.setFont(new Font("SansSerif",
				Font.PLAIN, 9));
		symbol.setFontColor(Color.BLACK);
		symbol.setFontSize(heightText);
		symbol.setStyle(FConstant.SYMBOL_STYLE_DGNSPECIAL);
		symbol.setSize(3);
		symbol.setSizeInPixels(true);
				
		int index = 0;		
		
		IntValue clave;
		FSymbol theSymbol = null;
					
		boolean setModel = false;	
		FeatureWithAttributes feature;
		try {
			while (iterator.hasNext()) {						
				try{	
					feature = (FeatureWithAttributes)iterator.next();
					if (feature != null){
						hashRelate.put(feature.getFeature().getID(), new Integer(index));
						features.add(feature);
						
						if (!setModel){
							getTableModel().setColumnIdentifiers(feature.getAttributeName());
						}
						clave = ValueFactory.createValue(index);
						IGeometry geom = feature.getFeature().getGeometry();
						if (geom != null){	
							FShape shape = (FShape)geom.getInternalShape();				
							
							theSymbol = new FSymbol(getShapeType());
							theSymbol.setDescription(clave.toString());
							theSymbol.setStyle(FConstant.SYMBOL_STYLE_DGNSPECIAL);
							theSymbol.setSize(3);
							theSymbol.setSizeInPixels(true);						
							
							addShape(shape, feature.getAttributeValue());	
						}
					}					
					index++;				
				}catch(GMLException e){
					logger.error(e.getMessage(),e);
				}
			}
//			ListBaseException listwarnings;
//			if(!warnings.isEmpty()){
//				System.out.println(warnings.getMessage());
//			}
			//Show Warnings...
//			if(reader.areWarnings()){
//				JDialog diag =  new JDialog();
//				diag.setTitle(Messages.getText("warning"));
//				diag.setSize(350, 210);
//				BasicWarningsPanel panel = new BasicWarningsPanel("GML:\n\n El parser intentar abrir el fichero\n\n Los datos pueden ser erroneos");
//				diag.getContentPane().add(panel);
//				diag.setResizable(false);
//		
//				//panel.printWarnings(warnings);
//				diag.show();
//			}
//			listwarnings = reader.getWarnings();
		}catch (GMLException e) {
			//e.setFilename(m_Fich.getName());
			logger.error(e.getMessage(),e);
		}
	}
	
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#accept(java.io.File)
	 */
	public boolean accept(File f) {
		if ((f.getName().toUpperCase().endsWith("GML")) ||
				(f.getName().toUpperCase().endsWith("XML"))){
			return true;
		}
		return false;
		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFile()
	 */
	public File getFile() {
		return m_Fich;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "gvSIG GML Memory Driver";
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		return FShape.MULTI;
	}

	/**
	 * @return Returns the hashRelate.
	 */
	public Hashtable getHashRelate() {
		return hashRelate;
	}
	
	/**
	 * @return Returns the features.
	 */
	public ArrayList getFeatures() {
		return features;
	}

	/**
	 * @return Returns the iterator.
	 */
	public IGMLFeaturesIterator getIterator() {
		return iterator;
	}	

	/**
	 * Gets the attributes list
	 * @return
	 */
	public String getAttributeName(int position){
		if (features.size() > 0){
			return ((String)((FeatureWithAttributes)features.get(0)).getAttributeName()[position]);
		}
		return null;
	}
	
	/**
	 * Gets the attributes list
	 * @return
	 */
	public Value getAttributeValue(int position){
		if (features.size() > 0){
			return ((Value)((FeatureWithAttributes)features.get(0)).getAttributeValue()[position]);
		}
		return null;
	}
	
	/**
	 * Gets the attributes size
	 * @return
	 */
	public int getAttributeCount(){
		if (features.size() > 0){
			return ((FeatureWithAttributes)features.get(0)).getAttributeValue().length;
		}
		return 0;
	}
}
