/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.print.attribute.PrintRequestAttributeSet;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.DriverNotLoadedExceptionType;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.WithDefaultLegend;
import com.iver.cit.gvsig.fmap.drivers.shp.IndexedShpDriver;
import com.iver.cit.gvsig.fmap.edition.AnnotationEditableAdapter;
import com.iver.cit.gvsig.fmap.edition.EditionEvent;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.ISpatialWriter;
import com.iver.cit.gvsig.fmap.edition.IWriteable;
import com.iver.cit.gvsig.fmap.edition.IWriter;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableDBAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Labelable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialXMLItem;
import com.iver.cit.gvsig.fmap.layers.layerOperations.XMLItem;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.StrategyManager;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo;
import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.SingleSymbolLegend;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.fmap.spatialindex.IPersistentSpatialIndex;
import com.iver.cit.gvsig.fmap.spatialindex.ISpatialIndex;
import com.iver.cit.gvsig.fmap.spatialindex.QuadtreeGt2;
import com.iver.cit.gvsig.fmap.spatialindex.QuadtreeJts;
import com.iver.cit.gvsig.fmap.spatialindex.SpatialIndexException;
import com.iver.utiles.FileUtils;
import com.iver.utiles.IPersistance;
import com.iver.utiles.PostProcessSupport;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.threads.Cancellable;
import com.iver.utiles.swing.threads.CancellableMonitorable;

/**
 * Capa bsica Vectorial.
 *
 * @author Fernando Gonzlez Corts
 */

// TODO Cuando no sea para pruebas debe no ser public
public class FLyrVect extends FLyrDefault implements Labelable,
		ClassifiableVectorial, SingleLayer, VectorialData, RandomVectorialData,
		AlphanumericData, InfoByPoint {
	private static Logger logger = Logger.getLogger(FLyrVect.class.getName());

	/** Leyenda de la capa vectorial */
	private VectorialLegend legend;

	private int typeShape = -1;

	private LayerChangeSupport layerChangeSupport = new LayerChangeSupport();

	private ReadableVectorial source;

	private SelectableDataSource sds;

	private SelectionSupport selectionSupport = new SelectionSupport();

	private SpatialCache spatialCache = new SpatialCache();
	private boolean spatialCacheEnabled = false;

	// protected SpatialIndex spatialIndex = null;
	/**
	 * An implementation of gvSIG spatial index
	 */
	protected ISpatialIndex spatialIndex = null;

	private boolean bHasJoin = false;

	private XMLEntity orgXMLEntity = null;

	private XMLEntity loadSelection = null;
	private VectorialLegend loadLegend = null;


	/**
	 * Aade un SelectionListener a la lista de listeners.
	 *
	 * @param listener
	 *            SelectionListener.
	 */
	/*
	 * public void addSelectionListener(SelectionListener listener) { try {
	 * getRecordset().addSelectionListener(listener); } catch (DriverException
	 * e) { // TODO Auto-generated catch block e.printStackTrace(); } }
	 */
	/**
	 * Borra un selectionListener de la lista de listeners.
	 *
	 * @param listener
	 *            SelectionListener
	 */
	/*
	 * public void removeSelectionListener(SelectionListener listener) { try {
	 * getRecordset().removeSelectionListener(listener); } catch
	 * (DriverException e) { // TODO Auto-generated catch block
	 * e.printStackTrace(); } }
	 */
	/**
	 * Cuando ocurre un evento de cambio en la seleccin, ste puede ser uno de
	 * una gran cantidad de eventos. Con el fin de no propagar todos estos
	 * eventos, se realiza la propagacin de manera manual al final de la
	 * "rfaga" de eventos
	 */
	/*
	 * public void fireSelectionEvents() { try {
	 * getRecordset().fireSelectionEvents(); } catch (DriverException e) { //
	 * TODO Auto-generated catch block e.printStackTrace(); } }
	 */
	/**
	 * Devuelve el VectorialAdapater de la capa.
	 *
	 * @return VectorialAdapter.
	 */
	public ReadableVectorial getSource() {
		if (!this.isAvailable()) return null;
		return source;
	}

	/**
	 * If we use a persistent spatial index associated with this layer, and the
	 * index is not intrisic to the layer (for example spatial databases) this
	 * method looks for existent spatial index, and loads it.
	 *
	 */
	private void loadSpatialIndex() {
		//FIXME: Al abrir el indice en fichero...
		//Cmo lo liberamos? un metodo Layer.shutdown()


		ReadableVectorial source = getSource();
		//REVISAR QUE PASA CON LOS DRIVERS DXF, DGN, etc.
		//PUES SON VECTORIALFILEADAPTER
		if (!(source instanceof VectorialFileAdapter)) {
			// we are not interested in db adapters
			return;
		}
		VectorialDriver driver = source.getDriver();
		if (!(driver instanceof BoundedShapes)) {
			// we dont spatially index layers that are not bounded
			return;
		}
		File file = ((VectorialFileAdapter) source).getFile();
		String fileName = file.getAbsolutePath();
		File sptFile = new File(fileName + ".qix");
		if (!sptFile.exists() || (!(sptFile.length() > 0))) {
			// before to exit, look for it in temp path
			String tempPath = System.getProperty("java.io.tmpdir");
			fileName = tempPath + File.separator + sptFile.getName();
			sptFile = new File(fileName);
			// it doesnt exists, must to create
			if (!sptFile.exists() || (!(sptFile.length() > 0))) {
				return;
			}// if
		}// if

		try {
			source.start();
			spatialIndex = new QuadtreeGt2(FileUtils.getFileWithoutExtension(sptFile),
					"NM", source.getFullExtent(), source.getShapeCount(), false);
		} catch (SpatialIndexException e) {
			spatialIndex = null;
			e.printStackTrace();
			return;
		} catch (DriverIOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * Checks if it has associated an external spatial index
	 * (an spatial index file).
	 *
	 * It looks for it in main file path, or in temp system path.
	 * If main file is rivers.shp, it looks for a file called
	 * rivers.shp.qix.

	 * @return
	 */
	public boolean isExternallySpatiallyIndexed() {
		/*
		 * FIXME (AZABALA): Independizar del tipo de fichero de ndice
	 	 * con el que se trabaje (ahora mismo considera la extension .qix,
	     * pero esto depender del tipo de ndice)
		 * */
		ReadableVectorial source = getSource();
		if (!(source instanceof VectorialFileAdapter)) {
			// we are not interested in db adapters.
			// think in non spatial dbs, like HSQLDB
			return false;
		}
		File file = ((VectorialFileAdapter) source).getFile();
		String fileName = file.getAbsolutePath();
		File sptFile = new File(fileName + ".qix");
		if (!sptFile.exists() || (!(sptFile.length() > 0))) {
			// before to exit, look for it in temp path
			// it doesnt exists, must to create
			String tempPath = System.getProperty("java.io.tmpdir");
			fileName = tempPath + File.separator + sptFile.getName();
			sptFile = new File(fileName);
			if (!sptFile.exists() || (!(sptFile.length() > 0))) {
				return false;
			}// if
		}// if
		return true;
	}

	/**
	 * Inserta el VectorialAdapter a la capa.
	 *
	 * @param va
	 *            VectorialAdapter.
	 */
	public void setSource(ReadableVectorial rv) {
		source = rv;
		// azabala: we check if this layer could have a file spatial index
		// and load it if it exists
		loadSpatialIndex();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws DriverException {
		try {
			Rectangle2D rAux;
//			logger.debug(getName() + "_source.start()");
			source.start();
			rAux = (Rectangle2D)source.getFullExtent().clone();
//			logger.debug(getName() + "_source.stop()");
			source.stop();

			// Si existe reproyeccin, reproyectar el extent
			ICoordTrans ct = getCoordTrans();

			if (ct != null) {
				Point2D pt1 = new Point2D.Double(rAux.getMinX(), rAux.getMinY());
				Point2D pt2 = new Point2D.Double(rAux.getMaxX(), rAux.getMaxY());
				pt1 = ct.convert(pt1, null);
				pt2 = ct.convert(pt2, null);
				rAux = new Rectangle2D.Double();
				rAux.setFrameFromDiagonal(pt1, pt2);
			}

			//Esto es para cuando se crea una capa nueva con el fullExtent de ancho y alto 0.
			if (rAux.getWidth()==0 && rAux.getHeight()==0) {
				rAux=new Rectangle2D.Double(0,0,100,100);
			}

			return rAux;
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#draw(java.awt.image.BufferedImage,
	 *      java.awt.Graphics2D, ISymbol)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
			Cancellable cancel, double scale) throws DriverException {

		if (isWithinScale(scale)) {
			// Las que solo tienen etiquetado sin pintar el shape,
			// no pasamos por ellas
			boolean bDrawShapes = true;
			if (legend instanceof SingleSymbolLegend) {
				if (legend.getDefaultSymbol().isShapeVisible() == false)
					bDrawShapes = false;
			}
			if (bDrawShapes) {
				Strategy strategy = StrategyManager.getStrategy(this);
				try {
					prepareDrawing(image, g, viewPort);
					strategy.draw(image, g, viewPort, cancel);
				} catch (DriverException e) {
					this.setVisible(false);
					this.setActive(false);
					throw e;
				}
			}
			if (getVirtualLayers() != null) {
				getVirtualLayers().draw(image, g, viewPort, cancel, scale);
			}

			if (getLayerText() != null) {
				getLayerText().draw(image, g, viewPort, cancel, scale);
			}
		}
	}

	/**
	 * Se llama antes de empezar a pintar.
	 * Es til para preparar la cache a emplear, las leyendas, etc.
	 * @param image
	 * @param g
	 * @param viewPort
	 */
	private void prepareDrawing(BufferedImage image, Graphics2D g, ViewPort viewPort) {

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#print(java.awt.Graphics2D,
	 *      com.iver.cit.gvsig.fmap.ViewPort,
	 *      com.iver.utiles.swing.threads.Cancellable)
	 */
	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel,
			double scale, PrintRequestAttributeSet properties) throws DriverException {
		if (isVisible() && isWithinScale(scale)) {
			Strategy strategy = StrategyManager.getStrategy(this);

			strategy.print(g, viewPort, cancel, properties);

			if (getLayerText() != null) {
				getLayerText().draw(null, g, viewPort, cancel, scale);
			}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#createLabelLayer(int)
	 */
	// public FLayer createLabelLayer(int fieldId) {
	public FLayer createLabelLayer(SelectableDataSource ds) {
		FLyrText layerText = null;
		try {
			layerText = new FLyrText();
			layerText.setCoordTrans(getCoordTrans());
			layerText.setLegend((VectorialLegend) getLegend());
			layerText.createLabels(this);
		} catch (FieldNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (DriverException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		setLayerText(layerText);
		return layerText;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#removeLabels()
	 */
	public void removeLabels() {
		setLayerText(null);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData#deleteSpatialIndex()
	 */
	public void deleteSpatialIndex() {
		//must we delete possible spatial indexes files?
		spatialIndex = null;
	}



   /**
    * <p>
    * Creates an spatial index associated to this layer.
    * The spatial index will used
    * the native projection of the layer, so if the layer is reprojected, it will
    * be ignored.
    * </p>
    * @param cancelMonitor instance of CancellableMonitorable that allows
    * to monitor progress of spatial index creation, and cancel the process
    */
    public void createSpatialIndex(CancellableMonitorable cancelMonitor){
    	 // FJP: ESTO HABR QUE CAMBIARLO. PARA LAS CAPAS SECUENCIALES, TENDREMOS
        // QUE ACCEDER CON UN WHILE NEXT. (O mejorar lo de los FeatureVisitor
        // para que acepten recorrer sin geometria, solo con rectangulos.

    	//If this vectorial layer is based in a spatial database, the spatial
		//index is already implicit. We only will index file drivers
    	ReadableVectorial va = getSource();
    	//We must think in non spatial databases, like HSQLDB
		if(!(va instanceof VectorialFileAdapter)){
			return;
		}
		if (!(va.getDriver() instanceof BoundedShapes)) {
			return;
		}
		File file = ((VectorialFileAdapter) va).getFile();
		String fileName = file.getAbsolutePath();
		ISpatialIndex localCopy = null;
		try {
			va.start();
			localCopy = new QuadtreeGt2(fileName, "NM", va.getFullExtent(),
					va.getShapeCount(), true);

		} catch (SpatialIndexException e1) {
			// Probably we dont have writing permissions
			String directoryName = System.getProperty("java.io.tmpdir");
			File newFile = new File(directoryName +
					File.separator +
					file.getName());
			String newFileName = newFile.getName();
			try {
				localCopy = new QuadtreeGt2(newFileName, "NM", va.getFullExtent(),
						va.getShapeCount(), true);
			} catch (SpatialIndexException e) {
				// if we cant build a file based spatial index, we'll build
				// a pure memory spatial index
				localCopy = new QuadtreeJts();
			} catch (DriverIOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}catch(Exception e){
			e.printStackTrace();
		}//try
        BoundedShapes shapeBounds = (BoundedShapes) va.getDriver();
        try {
            for (int i=0; i < va.getShapeCount(); i++)
            {
            	if(cancelMonitor != null){
            		if(cancelMonitor.isCanceled())
            			return;
            		cancelMonitor.reportStep();
            	}
                Rectangle2D r = shapeBounds.getShapeBounds(i);
                if(r != null)
                	localCopy.insert(r, i);
            } // for
            va.stop();
            if(localCopy instanceof IPersistentSpatialIndex)
            	((IPersistentSpatialIndex) localCopy).flush();
            spatialIndex = localCopy;
        } catch (DriverIOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData#createIndex()
	 */
	public void createSpatialIndex() {
		createSpatialIndex(null);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor,
	 *      FBitSet)
	 */
	public void process(FeatureVisitor visitor, FBitSet subset)
			throws DriverException, VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		s.process(visitor, subset);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor)
	 */
	public void process(FeatureVisitor visitor) throws DriverException,
			VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		s.process(visitor);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor,
	 *      Rectangle2D)
	 */
	public void process(FeatureVisitor visitor, Rectangle2D rect)
			throws DriverException, VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		s.process(visitor, rect);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#setSelection(com.iver.cit.gvsig.fmap.operations.selection.VectorialSubSet)
	 */
	/*
	 * public void setSelection(FBitSet selection) { try {
	 * getRecordset().setSelection(selection); } catch (DriverException e) { //
	 * TODO Auto-generated catch block e.printStackTrace(); }
	 * fireSelectionEvents(); }
	 */
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#isSelected(int)
	 */
	/*
	 * public boolean isSelected(int index) { try { return
	 * getRecordset().isSelected(index); } catch (DriverException e) { // TODO
	 * Auto-generated catch block e.printStackTrace(); } return false; }
	 */
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#getSelection()
	 */
	/*
	 * public FBitSet getSelection() { try { return
	 * getRecordset().getSelection(); } catch (DriverException e) { // TODO
	 * Auto-generated catch block e.printStackTrace(); } return new FBitSet(); }
	 */
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#clearSelection()
	 */
	/*
	 * public void clearSelection() { try { getRecordset().clearSelection(); }
	 * catch (DriverException e) { // TODO Auto-generated catch block
	 * e.printStackTrace(); } }
	 */

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData#queryByRect(java.awt.geom.Rectangle2D)
	 */
	public FBitSet queryByRect(Rectangle2D rect) throws DriverException {
		Strategy s = StrategyManager.getStrategy(this);

		return s.queryByRect(rect);
	}

	public FBitSet queryByPoint(Point2D p, double tolerance)
			throws DriverException {
		Strategy s = StrategyManager.getStrategy(this);
		return s.queryByPoint(p, tolerance);
	}

	public FBitSet queryByShape(IGeometry g, int relationship)
			throws DriverException, VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		return s.queryByShape(g, relationship);
	}

	public XMLItem[] getInfo(Point p, double tolerance, Cancellable cancel) throws DriverException {
		Point2D pReal = this.getMapContext().getViewPort().toMapPoint(p);
		FBitSet bs = queryByPoint(pReal, tolerance);
		VectorialXMLItem[] item = new VectorialXMLItem[1];
		item[0] = new VectorialXMLItem(bs, this);

		return item;
	}

	/**
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#getRecordset()
	 */
	/*
	 * public SelectableDataSource getRecordset() throws DriverException { if
	 * (sds == null){ try { DataSource ds = getSource().getRecordset();
	 *
	 * if (ds == null) { return null; }
	 *
	 * sds = new SelectableDataSource(ds);
	 * //sds.setSelectionSupport(selectionSupport);
	 *
	 * return sds; } catch (DriverLoadException e) { throw new
	 * DriverException(e); } catch
	 * (com.hardcode.gdbms.engine.data.driver.DriverException e) { throw new
	 * DriverException(e); } } return sds; }
	 */
	/**
	 * Para cuando haces una unin, sustituyes el recorset por el nuevo. De esta
	 * forma, podrs poner leyendas basadas en el nuevo recordset
	 *
	 * @param newSds
	 * @throws DriverException
	 * @throws FieldNotFoundException
	 * @throws FieldNotFoundException
	 */
	/*
	 * public void setRecordset(SelectableDataSource newSds) throws
	 * DriverException, FieldNotFoundException { sds = newSds;
	 * //sds.setSelectionSupport(selectionSupport); legend.setDataSource(sds);
	 * logger.debug("Recordset cambiado a " + sds.getName()); }
	 */
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#setLegend(int,
	 *      com.iver.cit.gvsig.fmap.rendering.Legend)
	 */
	public void setLegend(VectorialLegend r) throws DriverException,
			FieldNotFoundException {
		VectorialLegend oldLegend = legend;
		legend = r;

		try {
			legend.setDataSource(getRecordset());

			if (legend.getLabelField() != null) {
				// sds.start();
				// int idLabelField =
				// getRecordset().getFieldIndexByName(legend.getLabelField());
				createLabelLayer(getSource().getRecordset());
				// sds.stop();
			} else
				removeLabels();
		} catch (DriverException e) {
			throw new DriverException(e);
		} catch (FieldNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DriverLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		LegendChangedEvent e = LegendChangedEvent.createLegendChangedEvent(
				oldLegend, legend);
		callLegendChanged(e);
	}

	/**
	 * Devuelve la Leyenda de la capa.
	 *
	 * @return Leyenda.
	 */
	public Legend getLegend() {
		return legend;
	}

	/**
	 * Devuelve el tipo de shape que contiene la capa.
	 *
	 * @return tipo de shape.
	 *
	 * @throws DriverException
	 */
	public int getShapeType() throws DriverException {
		if (typeShape == -1) {
			try {
//				logger.debug("source.start()");
				getSource().start();
				typeShape = getSource().getShapeType();
//				logger.debug("source.stop()");
				getSource().stop();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
		}

		return typeShape;
	}

	/**
	 * @throws XMLException
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getProperties()
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		if (!this.isAvailable() && this.orgXMLEntity != null) {
			return this.orgXMLEntity;
		}
		XMLEntity xml = super.getXMLEntity();
		if (getLegend()!=null)
			xml.addChild(getLegend().getXMLEntity());
		try {
			if (getRecordset()!=null)
				xml.addChild(getRecordset().getSelectionSupport().getXMLEntity());
		} catch (DriverException e1) {
			e1.printStackTrace();
			throw new XMLException(e1);
		}
		//Repongo el mismo ReadableVectorial ms abajo para cuando se guarda el proyecto.
		ReadableVectorial rv=getSource();
		xml.putProperty("type", "vectorial");
		if (source instanceof VectorialEditableAdapter) {
				setSource(((VectorialEditableAdapter) source)
						.getOriginalAdapter());
			}
			if (source instanceof VectorialFileAdapter) {
				xml.putProperty("file", ((VectorialFileAdapter) source)
						.getFile());
				try {
					xml.putProperty("recordset-name", source
							.getRecordset().getName());
				} catch (DriverLoadException e) {
					e.printStackTrace();
				} catch (RuntimeException e) {
					e.printStackTrace();
				}
			} else if (source instanceof VectorialDBAdapter) {
				VectorialDatabaseDriver dbDriver = (VectorialDatabaseDriver) source
						.getDriver();

				// Guardamos el nombre del driver para poder recuperarlo
				// con el DriverManager de Fernando.
				xml.putProperty("db", dbDriver.getName());
				try {
					xml.putProperty("recordset-name", source
							.getRecordset().getName());
				} catch (DriverLoadException e) {
					e.printStackTrace();
				} catch (RuntimeException e) {
					e.printStackTrace();
				}
				if (dbDriver!=null)
					xml.addChild(dbDriver.getXMLEntity()); // Tercer child. Antes
														// hemos
				// metido la leyenda y el
				// selection support
			} else if (source instanceof VectorialAdapter) {
				VectorialDriver driver = (VectorialDriver) source.getDriver();

				// Guardamos el nombre del driver para poder recuperarlo
				// con el DriverManager de Fernando.
				xml.putProperty("other", driver.getName());
				try {
					xml.putProperty("recordset-name", source
							.getRecordset().getName());
				} catch (DriverLoadException e) {
					e.printStackTrace();
				} catch (RuntimeException e) {
					e.printStackTrace();
				}
				if (driver instanceof IPersistance) {
					// xml.putProperty("className",
					// driver.getClass().getName());
					IPersistance persist = (IPersistance) driver;
					xml.addChild(persist.getXMLEntity()); // Tercer child.
															// Antes
					// hemos metido la
					// leyenda y el
					// selection support
				}
			}
			if (rv!=null)
				setSource(rv);
			xml.putProperty("driverName", source.getDriver().getName());
	
		if (bHasJoin)
			xml.putProperty("hasJoin", "true");

		return xml;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity03(XMLEntity xml) throws XMLException {

		super.setXMLEntity(xml);
		legend = LegendFactory.createFromXML03(xml.getChild(0));

		try {
			// legend.setDataSource(getRecordset());
			setLegend(legend);
		} catch (FieldNotFoundException e) {
			throw new XMLException(e);
		} catch (DriverException e) {
			throw new XMLException(e);
		}

		try {
			getRecordset().getSelectionSupport()
					.setXMLEntity03(xml.getChild(1));
		} catch (DriverException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException {
		try {
			super.setXMLEntity(xml);

			VectorialLegend leg = LegendFactory.createFromXML(xml.getChild(0));
			try {
				getRecordset().getSelectionSupport().setXMLEntity(xml.getChild(1));
				/* JMVIVO
				this.setLoadSelection(xml.getChild(1));
				*/
				// JMVIVO: Esto sirve para algo????
				String recordsetName = xml.getStringProperty("recordset-name");

				LayerFactory.getDataSourceFactory().changeDataSourceName(
						getSource().getRecordset().getName(), recordsetName);
				// Hacemos que el recordset que hemos creado antes (los
				// recordset se crean antes que todo) sea el recordset
				// de esta capa. Y del que se crea automaticamente
				// en el setLegend del createLayer, nos olvidamos
//				 CHEMA: AUTOMATIC DATA SOURCE
				/*
				SelectableDataSource sds = new SelectableDataSource(LayerFactory
						.getDataSourceFactory().createRandomDataSource(
								recordsetName, DataSourceFactory.MANUAL_OPENING));
				*/
				SelectableDataSource sds = new SelectableDataSource(LayerFactory
						.getDataSourceFactory().createRandomDataSource(
								recordsetName, DataSourceFactory.AUTOMATIC_OPENING));
				// sds.setSelectionSupport(selectionSupport);
				// ((EditableAdapter)getSource()).setRecordSet(sds);
			} catch (NoSuchTableException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			} catch (DriverLoadException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			} catch (DriverException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			}
			// Si tiene una unin, lo marcamos para que no se cree la leyenda hasta
			// el final
			// de la lectura del proyecto
			if (xml.contains("hasJoin")) {
				setIsJoined(true);
				PostProcessSupport.addToPostProcess(this, "setLegend", leg, 1);
			} else {
				/*
				JMVIVO
				this.setLoadLegend(leg);
				*/
				try {
					// legend.setDataSource(getRecordset());
					setLegend(leg);
				} catch (FieldNotFoundException e) {
					throw new XMLException(e);
				} catch (DriverException e) {
					throw new XMLException(e);
				}
			}

		} catch (XMLException e) {
			this.setAvailable(false);
			this.orgXMLEntity = xml;
			//throw e;
		} catch (Exception e) {
			this.setAvailable(false);
			this.orgXMLEntity = xml;
			//throw new XMLException(e);
		}


	}

	public void setXMLEntityNew(XMLEntity xml) throws XMLException {
		try {
			super.setXMLEntity(xml);

			VectorialLegend leg = LegendFactory.createFromXML(xml.getChild(0));
			try {
				getRecordset().getSelectionSupport().setXMLEntity(xml.getChild(1));

				this.setLoadSelection(xml.getChild(1));
				/*
				// JMVIVO: Esto sirve para algo????
				String recordsetName = xml.getStringProperty("recordset-name");

				LayerFactory.getDataSourceFactory().changeDataSourceName(
						getSource().getRecordset().getName(), recordsetName);
				// Hacemos que el recordset que hemos creado antes (los
				// recordset se crean antes que todo) sea el recordset
				// de esta capa. Y del que se crea automaticamente
				// en el setLegend del createLayer, nos olvidamos
				SelectableDataSource sds = new SelectableDataSource(LayerFactory
						.getDataSourceFactory().createRandomDataSource(
								recordsetName, DataSourceFactory.AUTOMATIC_OPENING));
				// sds.setSelectionSupport(selectionSupport);
				// ((EditableAdapter)getSource()).setRecordSet(sds);


			} catch (NoSuchTableException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			*/
			} catch (DriverLoadException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			} catch (DriverException e1) {
				this.setAvailable(false);
				throw new XMLException(e1);
			}
			// Si tiene una unin, lo marcamos para que no se cree la leyenda hasta
			// el final
			// de la lectura del proyecto
			if (xml.contains("hasJoin")) {
				setIsJoined(true);
				PostProcessSupport.addToPostProcess(this, "setLegend", leg, 1);
			} else {


				this.setLoadLegend(leg);
				/*
				try {
					// legend.setDataSource(getRecordset());
					setLegend(leg);
				} catch (FieldNotFoundException e) {
					throw new XMLException(e);
				} catch (DriverException e) {
					throw new XMLException(e);
				}
				*/
			}

		} catch (XMLException e) {
			this.setAvailable(false);
			this.orgXMLEntity = xml;
			//throw e;
		} catch (Exception e) {
			this.setAvailable(false);
			this.orgXMLEntity = xml;
			//throw new XMLException(e);
		}


	}



	/**
	 * Aade un LegendListener a la lista de Listeners.
	 *
	 * @param listener
	 *            LegendListener.
	 */
	public void addLegendListener(LegendListener listener) {
		layerChangeSupport.addLayerListener(listener);
	}

	/**
	 * Llamada al mtodo callLegendChanged de los listener.
	 *
	 * @param e
	 *            Evento.
	 */
	private void callLegendChanged(LegendChangedEvent e) {
		layerChangeSupport.callLegendChanged(e);
	}

	/**
	 * Borra un LegendListener de la lista de Listeners
	 *
	 * @param listener
	 *            LegendListener.
	 */
	public void removeLegendListener(LegendListener listener) {
		layerChangeSupport.removeLayerListener(listener);
	}

	/**
	 * Sobreimplementacin del mtodo toString para que las bases de datos
	 * identifiquen la capa.
	 *
	 * @return DOCUMENT ME!
	 */
	public String toString() {
		/*
		 * Se usa internamente para que la parte de datos identifique de forma
		 * unvoca las tablas
		 */
		String ret = super.toString();

		return "layer" + ret.substring(ret.indexOf('@') + 1);
	}

	public boolean isJoined() {
		return bHasJoin;
	}

	/**
	 * Returns if a layer is spatially indexed
	 *
	 * @return if this layer has the ability to proces spatial queries without
	 *         secuential scans.
	 */
	public boolean isSpatiallyIndexed() {
		ReadableVectorial source = getSource();
		if (source instanceof ISpatialDB)
			return true;

//FIXME azabala
/*
 * Esto es muy dudoso, y puede cambiar.
 * Estoy diciendo que las que no son fichero o no son
 * BoundedShapes estan indexadas. Esto es mentira, pero
 * as quien pregunte no querr generar el indice.
 * Esta por ver si interesa generar el indice para capas
 * HSQLDB, WFS, etc.
 */
		if(!(source instanceof VectorialFileAdapter)){
			return true;
		}
		if (!(source.getDriver() instanceof BoundedShapes)) {
			return true;
		}

		if (getISpatialIndex() != null)
			return true;
		return false;
	}

	public void setIsJoined(boolean hasJoin) {
		bHasJoin = hasJoin;
	}

	/**
	 * @return Returns the spatialIndex.
	 */
	public ISpatialIndex getISpatialIndex() {
		return spatialIndex;
	}
	/**
	 * Sets the spatial index. This could be useful if, for some
	 * reasons, you want to work with a distinct spatial index
	 * (for example, a spatial index which could makes nearest
	 * neighbour querys)
	 * @param spatialIndex
	 */
	public void setISpatialIndex(ISpatialIndex spatialIndex){
		this.spatialIndex = spatialIndex;
	}


	/*
	 * public SelectableDataSource getRecordset() { try { return
	 * getSource().getRecordset(); } catch (DriverLoadException e) { // TODO
	 * Auto-generated catch block e.printStackTrace(); } return null; }
	 */

	/**
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#getRecordset()
	 */
	public SelectableDataSource getRecordset() throws DriverException {
		if (!this.isAvailable()) return null;
		if (sds == null) {
			try {
				SelectableDataSource ds = source.getRecordset();

				if (ds == null) {
					return null;
				}

				sds = ds;
				sds.setSelectionSupport(selectionSupport);

			} catch (DriverLoadException e) {
				DriverNotLoadedExceptionType exceptionType =
					new DriverNotLoadedExceptionType();
				exceptionType.setDriverName(source.getDriver().getName());
				throw new DriverException(e, exceptionType);
			}
		}
		return sds;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setEditing(boolean)
	 */
	public void setEditing(boolean b) throws EditionException {
		super.setEditing(b);
		try {
			if (b) {
				VectorialEditableAdapter vea = null;
				// TODO: Qu pasa si hay ms tipos de adapters?
				// FJP: Se podra pasar como argumento el
				// VectorialEditableAdapter
				// que se quiera usar para evitar meter cdigo aqu de este
				// estilo.
				if (getSource() instanceof VectorialDBAdapter) {
					vea = new VectorialEditableDBAdapter();
				} else if (this instanceof FLyrAnnotation) {
					vea = new AnnotationEditableAdapter(
							(FLyrAnnotation) this);
				} else {
					vea = new VectorialEditableAdapter();
				}
				vea.setOriginalVectorialAdapter(getSource());

				// /vea.setSpatialIndex(getSpatialIndex());
				// /vea.setFullExtent(getFullExtent());
				//CHEMA
				vea.start();
				//CHEMA
				vea.startEdition(EditionEvent.GRAPHIC);
				setSource(vea);
				getRecordset().setSelectionSupport(
						vea.getOriginalAdapter().getRecordset()
								.getSelectionSupport());

			} else {
				VectorialEditableAdapter vea = (VectorialEditableAdapter) getSource();
				//CHEMA
				vea.stop();
				//CHEMA
				setSource(vea.getOriginalAdapter());
			}
			// Si tenemos una leyenda, hay que pegarle el cambiazo a su
			// recordset
			setRecordset(getSource().getRecordset());
			if (getLegend() instanceof VectorialLegend) {
				VectorialLegend ley = (VectorialLegend) getLegend();
				ley.setDataSource(getSource().getRecordset());
				// Esto lo pongo para evitar que al dibujar sobre un
				// dxf, dwg, o dgn no veamos nada. Es debido al checkbox
				// de la leyenda de textos "dibujar solo textos".
				if (!(getSource().getDriver() instanceof IndexedShpDriver)){
					if (ley.getDefaultSymbol()==null){
						FSymbol symbol=new FSymbol(getShapeType());
						symbol.setFontSizeInPixels(false);
						symbol.setFont(new Font("SansSerif", Font.PLAIN, 9));
	//					Color color=symbol.getColor();
	//					int alpha=symbol.getColor().getAlpha();
	//					if (alpha>250) {
	//						symbol.setColor(new Color(color.getRed(),color.getGreen(),color.getBlue(),100));
//					}
					ley.setDefaultSymbol(symbol);
					}
				}
				ley.useDefaultSymbol(true);
			}
		} catch (DriverLoadException e) {
			e.printStackTrace();
			throw new EditionException(e);
		} catch (DriverException e) {
			e.printStackTrace();
			throw new EditionException(e);
		} catch (FieldNotFoundException e) {
			e.printStackTrace();
			throw new EditionException(e);
		//CHEMA
		} catch (DriverIOException e) {
			e.printStackTrace();
			throw new EditionException(e);
		//CHEMA
		}

		setSpatialCacheEnabled(b);
		callEditionChanged(LayerEvent
				.createEditionChangedEvent(this, "edition"));

	}

	/**
	 * Para cuando haces una unin, sustituyes el recorset por el nuevo. De esta
	 * forma, podrs poner leyendas basadas en el nuevo recordset
	 *
	 * @param newSds
	 */
	public void setRecordset(SelectableDataSource newSds) {
		sds = newSds;
		sds.setSelectionSupport(selectionSupport);
	}
	/*
	 * public SelectionSupport getSelectionSupport() { return selectionSupport; }
	 *
	 * public void setSelectionSupport(SelectionSupport selectionSupport) {
	 * this.selectionSupport = selectionSupport; }
	 */

//	public SpatialCache createSpatialCache() {
//       	spatialCache = new SpatialCache();
//		return spatialCache;
//	}
	public void clearSpatialCache()
	{
		spatialCache.clearAll();
	}

//	public void setSpatialCache(SpatialCache spatialCache) {
//		this.spatialCache = spatialCache;
//	}

	public boolean isSpatialCacheEnabled() {
		return spatialCacheEnabled;
	}

	public void setSpatialCacheEnabled(boolean spatialCacheEnabled) {
		this.spatialCacheEnabled = spatialCacheEnabled;
	}

	public SpatialCache getSpatialCache() {
		return spatialCache;
	}

	/**
	 * Siempre es un numero mayor de 1000
	 * @param maxFeatures
	 */
	public void setMaxFeaturesInEditionCache(int maxFeatures) {
		if (maxFeatures > spatialCache.maxFeatures)
			spatialCache.setMaxFeatures(maxFeatures);

	}

	/**
	 * This method returns a boolean that is used by the FPopMenu
	 * to make visible the properties menu or not. It is visible by
	 * default, and if a later don't have to show this menu only
	 * has to override this method.
	 * @return
	 * If the properties menu is visible (or not)
	 */
	public boolean isPropertiesMenuVisible(){
		return true;
	}

	public void reload() throws DriverIOException {
		super.reload();
		try {
			this.source.getDriver().reload();
			//this.getRecordset().reload();
			if (this.getLegend() == null) {
	            if (this.getRecordset().getDriver() instanceof WithDefaultLegend) {
	                WithDefaultLegend aux = (WithDefaultLegend) this.getRecordset().getDriver();
	                this.setLegend((VectorialLegend) aux.getDefaultLegend());
	            } else {
	                this.setLegend(LegendFactory.createSingleSymbolLegend(
	                        this.getShapeType()));
	            }
			}

		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			this.setAvailable(false);
			throw new DriverIOException(e);
		} catch (IOException e) {
			this.setAvailable(false);
			throw new DriverIOException(e);
		} catch (DriverException e) {
			this.setAvailable(false);
			throw new DriverIOException(e);
		} catch (Exception e) {
			this.setAvailable(false);
			throw new DriverIOException(e);
		}

	}

	protected void setLoadSelection(XMLEntity xml) {
		this.loadSelection = xml;
	}

	protected void setLoadLegend(VectorialLegend legend) {
		this.loadLegend = legend;
	}

	protected void putLoadSelection() throws XMLException {
		if (this.loadSelection == null) return;
		try {
			this.getRecordset().getSelectionSupport().setXMLEntity(this.loadSelection);
		} catch (DriverException e) {
			throw new XMLException(e);
		}
		this.loadSelection = null;

	}
	protected void putLoadLegend() throws FieldNotFoundException, DriverException {
		if (this.loadLegend == null) return;

		this.setLegend(this.loadLegend);

		this.loadLegend = null;

	}

	protected void cleanLoadOptions() {
		this.loadLegend = null;
		this.loadSelection = null;
	}

	public boolean isWritable() {
		VectorialDriver drv = getSource().getDriver();
		if (!drv.isWritable())
			return false;
//		VectorialEditableAdapter vea = (VectorialEditableAdapter) lyrVect
//				.getSource();
		if (drv instanceof IWriteable)
		{
			IWriter writer = ((IWriteable)drv).getWriter();
			if (writer != null)
			{
				if (writer instanceof ISpatialWriter)
					return true;
			}
		}
		return false;

	}

	public void beforePrinting(PrintRequestAttributeSet properties) {
		VectorialLegend vl=(VectorialLegend)getLegend();
		if (vl instanceof ClassifiedLegendInfo) {
			ClassifiedLegendInfo clsfLegend = (ClassifiedLegendInfo) vl;
			ISymbol[] symbols=clsfLegend.getSymbols();
			for (int i=0;i<symbols.length;i++) {
				symbols[i].setPrintingProperties(properties);
			}
		}
		if (vl.getDefaultSymbol()!=null) {
			vl.getDefaultSymbol().setPrintingProperties(properties);
		}
	}

	public void afterPrinting() {
		VectorialLegend vl=(VectorialLegend)getLegend();
		if (vl instanceof ClassifiedLegendInfo) {
			ClassifiedLegendInfo clsfLegend = (ClassifiedLegendInfo) vl;
			ISymbol[] symbols=clsfLegend.getSymbols();
			for (int i=0;i<symbols.length;i++) {
				symbols[i].setPrintingProperties(null);
			}
		}
		if (vl.getDefaultSymbol()!=null) {
			vl.getDefaultSymbol().setPrintingProperties(null);
		}

	}

	public FLayer cloneLayer() throws Exception {
		FLyrVect clonedLayer = new FLyrVect();
		clonedLayer.setSource(getSource());
		clonedLayer.setVisible(isVisible());
		clonedLayer.setISpatialIndex(getISpatialIndex());
		clonedLayer.setName(getName());
		clonedLayer.setCoordTrans(getCoordTrans());

		if (legend.getLabelField() != null) // Hay etiquetas
		{
			FLyrText lyrText = getLayerText();
			String antLabelField = legend.getLabelField();
			VectorialLegend clonedLegend = (VectorialLegend) legend.cloneLegend();
			clonedLegend.setLabelField(null);
			clonedLayer.setLegend(clonedLegend);
			clonedLayer.setLayerText(lyrText);
			clonedLegend.setLabelField(antLabelField);

		}
		else
			clonedLayer.setLegend((VectorialLegend)getLegend());
		return clonedLayer;
	}

	/**
	 * This method tells the driver that the layer is going to be removed
	 * from the view, if it is a VectorialDatabaseDriver, because those
	 * drivers can have problems if that event is not notified.
	 * This overwrites FLyrDefault.removingThisLayer().
	 */
	public void removingThisLayer() {
		if (sds==null)
			return;
		Driver drv = sds.getDriver();
		if (drv != null) {
			if (drv instanceof VectorialDatabaseDriver) {
				((VectorialDatabaseDriver) drv).remove();
			}
		} else {
			logger.warn("Unable to send removing event to driver (driver is null)");
		}

	}

}
