/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import java.util.ArrayList;


/**
 * Behaviour que espera un listener de tipo MeasureListener.
 *
 * @author Vicente Caballero Navarro
 */
public class PolylineBehavior extends Behavior {
	protected ArrayList arrayX = new ArrayList();
	protected ArrayList arrayY = new ArrayList();
	protected boolean isClicked = false;
	protected PolylineListener listener;

	/**
	 * Crea un nuevo PolylineBehavior.
	 *
	 * @param mli listener.
	 */
	public PolylineBehavior(PolylineListener mli) {
		listener = mli;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		g.drawImage(getMapControl().getImage(), 0, 0, null);
		g.setColor(Color.black);

		if (!arrayX.isEmpty()) {
			drawPolyLine((Graphics2D) g);
		}
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior.
	 *
	 * @param E MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mousePressed(MouseEvent E) throws BehaviorException {
		if (E.getClickCount() == 2) {
			//System.err.println("dobleclick");
			//Point2D p1=getMapControl().getViewPort().toMapPoint();
			listener.polylineFinished(new MeasureEvent(
					(Double[]) arrayX.toArray(new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E));
			arrayX.clear();
			arrayY.clear();
			isClicked = false;
		} else {
			//System.err.println("simpleclick");
			isClicked = true;
			addPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));

			if (arrayX.size() < 2) {
				addPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));
			}

			listener.pointFixed(new MeasureEvent(
					(Double[]) arrayX.toArray(new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E));
		}
	}

	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		mouseMoved(e);
	}

	/**
	 * Cambio del ltimo punto.
	 *
	 * @param p punto.
	 */
	protected void changeLastPoint(Point2D p) {
		arrayX.set(arrayX.size() - 1, new Double(p.getX()));
		arrayY.set(arrayY.size() - 1, new Double(p.getY()));
	}

	/**
	 * Reimplementacin del mtodo mouseMoved de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseMoved(MouseEvent E) throws BehaviorException {
		//System.err.println("moved antes de click");
		if (isClicked) {
			//System.err.println("moved despues de click");
			changeLastPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));

			MeasureEvent event = new MeasureEvent((Double[]) arrayX.toArray(
						new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E);
			listener.points(event);
			getMapControl().repaint();
		}
	}

	/**
	 * Dibujo de la polilnea.
	 *
	 * @param g2 Graphics2D sobre el que dibujamos.
	 */
	protected void drawPolyLine(Graphics2D g2) {
		GeneralPathX polyline = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
				arrayX.size());
		Point2D pScreen0=getMapControl().getViewPort().fromMapPoint(new Point2D.Double(((Double) arrayX.get(0)).doubleValue(),
				((Double) arrayY.get(0)).doubleValue()));
		polyline.moveTo(pScreen0.getX(),
				pScreen0.getY());

		for (int index = 1; index < arrayX.size(); index++) {
			Point2D pScreen=getMapControl().getViewPort().fromMapPoint(new Point2D.Double(((Double) arrayX.get(index)).doubleValue(),
					((Double) arrayY.get(index)).doubleValue()));
			polyline.lineTo(pScreen.getX(),pScreen.getY());
		}

		g2.draw(polyline);
	}

	/**
	 * Aade un punto a la polilnea.
	 *
	 * @param p Punto.
	 */
	protected void addPoint(Point2D p) {
		arrayX.add(new Double(p.getX()));
		arrayY.add(new Double(p.getY()));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(com.iver.cit.gvsig.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (PolylineListener) listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
}
