package com.hardcode.gdbms.engine.data.driver;

import java.io.File;
import java.io.IOException;

import com.hardcode.driverManager.Driver;
import com.hardcode.gdbms.engine.data.file.FileDataWare;


/**
 * Interface to the files.
 */
public interface FileDriver extends ReadAccess, GDBMSDriver {
    /**
     * Mtodo invocado al comienzo para abrir el fichero. A partir de la
     * invocacin de este mtodo todos las operaciones se harn sobre el File
     * que se pasa como parmetro
     *
     * @param file Fichero que se debe de abrir
     *
     * @throws IOException Si se produce algun error
     */
    void open(File file) throws IOException;

    /**
     * Cierra el Fichero sobre el que se estaba accediendo
     *
     * @throws IOException Si se produce algn error
     */
    void close() throws IOException;

    /**
     * devuelve true si el driver puede leer el fichero que se pasa como
     * parmetro, false en caso contrario
     *
     * @param f Fichero que se quiere comprobar
     *
     * @return DOCUMENT ME!
     */
    boolean fileAccepted(File f);

    /**
     * Writes the content in the DataWare to the last opened file (method open). 
     * The data must be written to a buffer (memory or disk) and then copied 
     * to the file because if the file is modified the dataWare data may be 
     * corrupted. A temporary file can be obtained from the DataSourceFactory
     * passed to the setDataSourceFactory method to delegate clean tasks to the
     * system
     *
     * @param dataWare DataWare with the contents
     */
    void writeFile(FileDataWare dataWare) throws DriverException;
    
    /**
     * Creates a new file with the given field names and types
     * 
     * @param path Path to the new file
     * @param fieldNames Names of the fields
     * @param fieldTypes Types of the fields
     * 
     * @throws IOException if the source cannot be created
     */
    void createSource(String path, String[] fieldNames, int[] fieldTypes) throws IOException;
}
